@@ -28,10 +28,6 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #ifndef IPMI_INTF_H
@@ -68,7 +64,7 @@ enum LANPLUS_SESSION_STATE {
 
 struct ipmi_session {
 	uint8_t hostname[64];
-	uint8_t username[16];
+	uint8_t username[17];
 	uint8_t authcode[IPMI_AUTHCODE_BUFFER_SIZE + 1];
 	uint8_t challenge[16];
 	uint8_t authtype;
@@ -82,6 +78,7 @@ struct ipmi_session {
 	uint8_t authextra;
 	uint8_t privlvl;
 	uint8_t cipher_suite_id;
+	char sol_escape_char;
 	int password;
 	int port;
 	int active;
@@ -173,6 +170,8 @@ struct ipmi_intf {
 	struct ipmi_cmd * cmdlist;
 	uint32_t my_addr;
 	uint32_t target_addr;
+	uint8_t target_lun;
+	uint8_t target_channel;
 
 	int (*setup)(struct ipmi_intf * intf);
 	int (*open)(struct ipmi_intf * intf);
@@ -192,6 +191,8 @@ void ipmi_intf_session_set_username(struct ipmi_intf * intf, char * username);
 void ipmi_intf_session_set_password(struct ipmi_intf * intf, char * password);
 void ipmi_intf_session_set_privlvl(struct ipmi_intf * intf, uint8_t privlvl);
 void ipmi_intf_session_set_cipher_suite_id(struct ipmi_intf * intf, uint8_t cipher_suite_id);
+void ipmi_intf_session_set_sol_escape_char(struct ipmi_intf * intf, char sol_escape_char);
+void ipmi_intf_session_set_kgkey(struct ipmi_intf * intf, char * kgkey);
 void ipmi_intf_session_set_port(struct ipmi_intf * intf, int port);
 void ipmi_intf_session_set_authtype(struct ipmi_intf * intf, uint8_t authtype);
 void ipmi_intf_session_set_timeout(struct ipmi_intf * intf, uint32_t timeout);
