@@ -28,24 +28,30 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/ioctl.h>  /* For TIOCNOTTY */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <inttypes.h>
 #include <signal.h>
 #include <string.h>
 #include <strings.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
 
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#ifdef HAVE_PATHS_H
+# include <paths.h>  /* For _PATH_TTY */
+#endif
+
 #include <ipmitool/helper.h>
 #include <ipmitool/log.h>
 
@@ -100,34 +106,129 @@ void printbuf(const uint8_t * buf, int len, const char * desc)
 
 const char * val2str(uint16_t val, const struct valstr *vs)
 {
-	static char un_str[16];
-	int i = 0;
+	static char un_str[32];
+	int i;
 
-	while (vs[i].str != NULL) {
+	for (i = 0; vs[i].str != NULL; i++) {
 		if (vs[i].val == val)
 			return vs[i].str;
-		i++;
 	}
 
-	memset(un_str, 0, 16);
-	snprintf(un_str, 16, "Unknown (0x%02x)", val);
+	memset(un_str, 0, 32);
+	snprintf(un_str, 32, "Unknown (0x%x)", val);
 
 	return un_str;
 }
 
 uint16_t str2val(const char *str, const struct valstr *vs)
 {
-	int i = 0;
+	int i;
 
-	while (vs[i].str != NULL) {
+	for (i = 0; vs[i].str != NULL; i++) {
 		if (strncasecmp(vs[i].str, str, __maxlen(str, vs[i].str)) == 0)
 			return vs[i].val;
-		i++;
 	}
 
 	return vs[i].val;
 }
 
+/* print_valstr  -  print value string list to log or stdout
+ *
+ * @vs:		value string list to print
+ * @title:	name of this value string list
+ * @loglevel:	what log level to print, -1 for stdout
+ */
+void
+print_valstr(const struct valstr * vs, const char * title, int loglevel)
+{
+	int i;
+
+	if (vs == NULL)
+		return;
+
+	if (title != NULL) {
+		if (loglevel < 0)
+			printf("\n%s:\n\n", title);
+		else
+			lprintf(loglevel, "\n%s:\n", title);
+	}
+
+	if (loglevel < 0) {
+		printf("  VALUE\tHEX\tSTRING\n");
+		printf("==============================================\n");
+	} else {
+		lprintf(loglevel, "  VAL\tHEX\tSTRING");
+		lprintf(loglevel, "==============================================");
+	}
+
+	for (i = 0; vs[i].str != NULL; i++) {
+		if (loglevel < 0) {
+			if (vs[i].val < 256)
+				printf("  %d\t0x%02x\t%s\n", vs[i].val, vs[i].val, vs[i].str);
+			else
+				printf("  %d\t0x%04x\t%s\n", vs[i].val, vs[i].val, vs[i].str);
+		} else {
+			if (vs[i].val < 256)
+				lprintf(loglevel, "  %d\t0x%02x\t%s", vs[i].val, vs[i].val, vs[i].str);
+			else
+				lprintf(loglevel, "  %d\t0x%04x\t%s", vs[i].val, vs[i].val, vs[i].str);
+		}
+	}
+
+	if (loglevel < 0)
+		printf("\n");
+	else
+		lprintf(loglevel, "");
+}
+
+/* print_valstr_2col  -  print value string list in two columns to log or stdout
+ *
+ * @vs:		value string list to print
+ * @title:	name of this value string list
+ * @loglevel:	what log level to print, -1 for stdout
+ */
+void
+print_valstr_2col(const struct valstr * vs, const char * title, int loglevel)
+{
+	int i;
+
+	if (vs == NULL)
+		return;
+
+	if (title != NULL) {
+		if (loglevel < 0)
+			printf("\n%s:\n\n", title);
+		else
+			lprintf(loglevel, "\n%s:\n", title);
+	}
+
+	for (i = 0; vs[i].str != NULL; i++) {
+		if (vs[i+1].str == NULL) {
+			/* last one */
+			if (loglevel < 0) {
+				printf("  %4d  %-32s\n", vs[i].val, vs[i].str);
+			} else {
+				lprintf(loglevel, "  %4d  %-32s\n", vs[i].val, vs[i].str);
+			}
+		}
+		else {
+			if (loglevel < 0) {
+				printf("  %4d  %-32s    %4d  %-32s\n",
+				       vs[i].val, vs[i].str, vs[i+1].val, vs[i+1].str);
+			} else {
+				lprintf(loglevel, "  %4d  %-32s    %4d  %-32s\n",
+					vs[i].val, vs[i].str, vs[i+1].val, vs[i+1].str);
+			}
+			i++;
+		}
+	}
+
+	if (loglevel < 0)
+		printf("\n");
+	else
+		lprintf(loglevel, "");
+}
+
 /* ipmi_csum  -  calculate an ipmi checksum
  *
  * @d:		buffer to check
@@ -174,6 +275,18 @@ ipmi_open_file(const char * file, int rw)
 		}
 	}
 
+#ifndef ENABLE_FILE_SECURITY
+	if (!rw) {
+		/* on read skip the extra checks */
+		fp = fopen(file, "r");
+		if (fp == NULL) {
+			lperror(LOG_ERR, "Unable to open file %s", file);
+			return NULL;
+		}
+		return fp;
+	}
+#endif
+
 	/* it exists - only regular files, not links */
 	if (S_ISREG(st1.st_mode) == 0) {
 		lprintf(LOG_ERR, "File %s has invalid mode: %d",
