@@ -28,21 +28,17 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
+#include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <errno.h>
-#include <sys/types.h>
 #include <unistd.h>
 #include <signal.h>
 #include <setjmp.h>
@@ -59,6 +55,123 @@
 
 extern int verbose;
 
+/* is_lan_channel - Check if channel is LAN medium
+ *
+ * return 1 if channel is LAN
+ * return 0 if channel is not LAN
+ *
+ * @intf:    ipmi interface handle
+ * @chan:    channel number to check
+ */
+static int
+is_lan_channel(struct ipmi_intf * intf, uint8_t chan)
+{
+	uint8_t medium;
+
+	if (chan < 1 || chan > IPMI_CHANNEL_NUMBER_MAX)
+		return 0;
+
+	medium = ipmi_get_channel_medium(intf, chan);
+
+	if (medium == IPMI_CHANNEL_MEDIUM_LAN ||
+	    medium == IPMI_CHANNEL_MEDIUM_LAN_OTHER)
+		return 1;
+
+	return 0;
+}
+
+/* find_lan_channel - Find first channel that is LAN
+ *
+ * return channel number if successful
+ * return -1 if no lan channel found
+ *
+ * @intf:    ipmi interface handle
+ * @start:   channel number to start searching from
+ */
+static int
+find_lan_channel(struct ipmi_intf * intf, int start)
+{
+	uint8_t chan;
+
+	for (chan = start; chan < IPMI_CHANNEL_NUMBER_MAX; chan++) {
+		if (is_lan_channel(intf, chan)) {
+			return chan;
+		}
+	}
+	return -1;
+}
+
+/* get_lan_param_select - Query BMC for LAN parameter data
+ *
+ * return pointer to lan_param if successful
+ * if parameter not supported then
+ *   return pointer to lan_param with
+ *   lan_param->data == NULL and lan_param->data_len == 0
+ * return NULL on error
+ *
+ * @intf:    ipmi interface handle
+ * @chan:    ipmi channel
+ * @param:   lan parameter id
+ * @select:  lan parameter set selector
+ */
+static struct lan_param *
+get_lan_param_select(struct ipmi_intf * intf, uint8_t chan, int param, int select)
+{
+	struct lan_param * p;
+	struct ipmi_rs * rsp;
+	struct ipmi_rq req;
+	uint8_t msg_data[4];
+
+	p = &ipmi_lan_params[param];
+	if (p == NULL)
+		return NULL;
+
+	msg_data[0] = chan;
+	msg_data[1] = p->cmd;
+	msg_data[2] = select;
+	msg_data[3] = 0;
+
+	memset(&req, 0, sizeof(req));
+	req.msg.netfn    = IPMI_NETFN_TRANSPORT;
+	req.msg.cmd      = IPMI_LAN_GET_CONFIG;
+	req.msg.data     = msg_data;
+	req.msg.data_len = 4;
+
+	rsp = intf->sendrecv(intf, &req);
+	if (rsp == NULL) {
+		lprintf(LOG_INFO, "Get LAN Parameter '%s' command failed", p->desc);
+		return NULL;
+	}
+
+	switch (rsp->ccode)
+	{
+	case 0x00: /* successful */
+		break;
+
+	case 0x80: /* parameter not supported */
+	case 0xc9: /* parameter out of range */
+	case 0xcc: /* invalid data field in request */
+
+		/* these completion codes usually mean parameter not supported */
+		lprintf(LOG_INFO, "Get LAN Parameter '%s' command failed: %s",
+			p->desc, val2str(rsp->ccode, completion_code_vals));
+		p->data = NULL;
+		p->data_len = 0;
+		return p;
+
+	default:
+
+		/* other completion codes are treated as error */
+		lprintf(LOG_INFO, "Get LAN Parameter '%s' command failed: %s",
+			p->desc, val2str(rsp->ccode, completion_code_vals));
+		return NULL;
+	}
+
+	p->data = rsp->data + 1;
+	p->data_len = rsp->data_len - 1;
+
+	return p;
+}
 
 /* get_lan_param - Query BMC for LAN parameter data
  *
@@ -75,57 +188,7 @@ extern int verbose;
 static struct lan_param *
 get_lan_param(struct ipmi_intf * intf, uint8_t chan, int param)
 {
-	struct lan_param * p;
-	struct ipmi_rs * rsp;
-	struct ipmi_rq req;
-	uint8_t msg_data[4];
-
-	p = &ipmi_lan_params[param];
-	if (p == NULL)
-		return NULL;
-
-	msg_data[0] = chan;
-	msg_data[1] = p->cmd;
-	msg_data[2] = 0;
-	msg_data[3] = 0;
-
-	memset(&req, 0, sizeof(req));
-	req.msg.netfn    = IPMI_NETFN_TRANSPORT;
-	req.msg.cmd      = IPMI_LAN_GET_CONFIG;
-	req.msg.data     = msg_data;
-	req.msg.data_len = 4;
-
-	rsp = intf->sendrecv(intf, &req);
-	if (rsp == NULL) {
-		lprintf(LOG_INFO, "Get LAN Parameter command failed");
-		return NULL;
-	}
-	if (rsp->ccode == 0x80) {
-		/* parameter not supported
-		 * return lan_param without data
-		 */
-		p->data = NULL;
-		p->data_len = 0;
-		return p;
-	}
-	if (rsp->ccode == 0xc9) {
-		/* parameter out of range
-		 * return lan_param without data
-		 */
-		p->data = NULL;
-		p->data_len = 0;
-		return p;
-	}
-	if (rsp->ccode > 0) {
-		lprintf(LOG_INFO, "Get LAN Parameter command failed: %s",
-			val2str(rsp->ccode, completion_code_vals));
-		return NULL;
-	}
-
-	p->data = rsp->data + 1;
-	p->data_len = rsp->data_len - 1;
-
-	return p;
+	return get_lan_param_select(intf, chan, param, 0);
 }
 
 /* set_lan_param_wait - Wait for Set LAN Parameter command to complete
@@ -276,6 +339,8 @@ ipmi_lanp_lock_state(struct ipmi_intf * intf, uint8_t chan)
 	p = get_lan_param(intf, chan, IPMI_LANP_SET_IN_PROGRESS);
 	if (p == NULL)
 		return -1;
+	if (p->data == NULL)
+		return -1;
 	return (p->data[0] & 3);
 }
 
@@ -355,6 +420,26 @@ set_lan_param(struct ipmi_intf * intf, uint8_t chan,
 	return rc;
 }
 
+/* set_lan_param_nowait - Wrap LAN parameter write without set-in-progress lock
+ *
+ * Returns value from __set_lan_param()
+ *
+ * @intf:    ipmi interface handle
+ * @chan:    ipmi channel
+ * @param:   lan parameter id
+ * @data:    lan parameter data
+ * @len:     length of lan parameter data
+ */
+static int
+set_lan_param_nowait(struct ipmi_intf * intf, uint8_t chan,
+		     int param, uint8_t * data, int len)
+{
+	int rc;
+	ipmi_lanp_lock(intf, chan);
+	rc = __set_lan_param(intf, chan, param, data, len, 0);
+	ipmi_lanp_unlock(intf, chan);
+	return rc;
+}
 
 static int
 lan_set_arp_interval(struct ipmi_intf * intf,
@@ -371,7 +456,7 @@ lan_set_arp_interval(struct ipmi_intf * intf,
 		return -1;
 
 	if (ival != 0) {
-		interval = ((uint8_t)atoi(ival) * 2) - 1;
+		interval = ((uint8_t)atoi((const char *)ival) * 2) - 1;
 		rc = set_lan_param(intf, chan, IPMI_LANP_GRAT_ARP, &interval, 1);
 	} else {
 		interval = lp->data[0];
@@ -463,7 +548,6 @@ static int
 ipmi_lan_print(struct ipmi_intf * intf, uint8_t chan)
 {
 	struct lan_param * p;
-	uint8_t medium;
 	int rc = 0;
 
 	if (chan < 1 || chan > IPMI_CHANNEL_NUMBER_MAX) {
@@ -472,11 +556,8 @@ ipmi_lan_print(struct ipmi_intf * intf, uint8_t chan)
 	}
 
 	/* find type of channel and only accept 802.3 LAN */
-	medium = ipmi_get_channel_medium(intf, chan);
-	if (medium != IPMI_CHANNEL_MEDIUM_LAN &&
-	    medium != IPMI_CHANNEL_MEDIUM_LAN_OTHER) {
-		lprintf(LOG_ERR, "Channel %d (%s) is not a LAN channel",
-			chan, val2str(medium, ipmi_channel_medium_vals), medium);
+	if (is_lan_channel(intf, chan) == 0) {
+		lprintf(LOG_ERR, "Channel %d is not a LAN channel", chan);
 		return -1;
 	}
 
@@ -652,7 +733,18 @@ ipmi_lan_print(struct ipmi_intf * intf, uint8_t chan)
 		printf("%-24s: %02x:%02x:%02x:%02x:%02x:%02x\n", p->desc,
 		       p->data[0], p->data[1], p->data[2], p->data[3], p->data[4], p->data[5]);
 
+	p = get_lan_param(intf, chan, IPMI_LANP_VLAN_ID);
+	if (p != NULL && p->data != NULL) {
+		int id = ((p->data[1] & 0x0f) << 8) + p->data[0];
+		if (p->data[1] & 0x80)
+			printf("%-24s: %d\n", p->desc, id);
+		else
+			printf("%-24s: Disabled\n", p->desc);
+	}
 
+	p = get_lan_param(intf, chan, IPMI_LANP_VLAN_PRIORITY);
+	if (p != NULL && p->data != NULL)
+		printf("%-24s: %d\n", p->desc, p->data[0] & 0x07);
 
 	/* Determine supported Cipher Suites -- Requires two calls */
 	p = get_lan_param(intf, chan, IPMI_LANP_RMCP_CIPHER_SUPPORT);
@@ -805,7 +897,7 @@ ipmi_lan_set_password(struct ipmi_intf * intf,
 	data[1] = 0x02;		/* set password */
 
 	if (password != NULL)
-		memcpy(data+2, password, __min(strlen(password), 16));
+		memcpy(data+2, password, __min(strlen((const char *)password), 16));
 
 	memset(&req, 0, sizeof(req));
 	req.msg.netfn = IPMI_NETFN_APP;
@@ -827,13 +919,85 @@ ipmi_lan_set_password(struct ipmi_intf * intf,
 	/* adjust our session password
 	 * or we will no longer be able to communicate with BMC
 	 */
-	ipmi_intf_session_set_password(intf, password);
+	ipmi_intf_session_set_password(intf, (char *)password);
 	printf("Password %s for user %d\n",
 	       (password == NULL) ? "cleared" : "set", userid);
 
 	return 0;
 }
 
+static int
+ipmi_set_alert_enable(struct ipmi_intf * intf, uint8_t channel, uint8_t enable)
+{
+	struct ipmi_rs * rsp;
+	struct ipmi_rq req;
+	uint8_t rqdata[3];
+
+	memset(&req, 0, sizeof(req));
+
+	/* update non-volatile access */
+	rqdata[0] = channel;
+	rqdata[1] = 0x40;
+
+	req.msg.netfn = IPMI_NETFN_APP;
+	req.msg.cmd = 0x41;
+	req.msg.data = rqdata;
+	req.msg.data_len = 2;
+
+	rsp = intf->sendrecv(intf, &req);
+	if (rsp == NULL) {
+		lprintf(LOG_ERR, "Unable to Get Channel Access for channel %d", channel);
+		return -1;
+	}
+	if (rsp->ccode > 0) {
+		lprintf(LOG_ERR, "Get Channel Access for channel %d failed: %s",
+			channel, val2str(rsp->ccode, completion_code_vals));
+		return -1;
+	}
+
+	/* SAVE TO NVRAM */
+	memset(rqdata, 0, 3);
+	rqdata[0] = channel & 0xf;
+	rqdata[1] = rsp->data[0];
+	if (enable != 0)
+		rqdata[1] &= ~0x20;
+	else
+		rqdata[1] |= 0x20;
+	rqdata[1] |= 0x40;
+	rqdata[2] = 0;
+
+	req.msg.cmd = 0x40;
+	req.msg.data_len = 3;
+
+	rsp = intf->sendrecv(intf, &req);
+	if (rsp == NULL) {
+		lprintf(LOG_ERR, "Unable to Set Channel Access for channel %d", channel);
+		return -1;
+	}
+	if (rsp->ccode > 0) {
+		lprintf(LOG_ERR, "Set Channel Access for channel %d failed: %s",
+			channel, val2str(rsp->ccode, completion_code_vals));
+		return -1;
+	}
+
+	/* SAVE TO CURRENT */
+	rqdata[1] &= 0xc0;
+	rqdata[1] |= 0x80;
+
+	rsp = intf->sendrecv(intf, &req);
+	if (rsp == NULL) {
+		lprintf(LOG_ERR, "Unable to Set Channel Access for channel %d", channel);
+		return -1;
+	}
+	if (rsp->ccode > 0) {
+		lprintf(LOG_ERR, "Set Channel Access for channel %d failed: %s",
+			channel, val2str(rsp->ccode, completion_code_vals));
+		return -1;
+	}
+
+	return 0;
+}
+
 static int
 ipmi_set_channel_access(struct ipmi_intf * intf, uint8_t channel, uint8_t enable)
 {
@@ -851,7 +1015,7 @@ ipmi_set_channel_access(struct ipmi_intf * intf, uint8_t channel, uint8_t enable
 	/* SAVE TO NVRAM */
 	memset(rqdata, 0, 3);
 	rqdata[0] = channel & 0xf;
-	rqdata[1] = 0x60;	/* set pef disabled, per-msg auth enabled */
+	rqdata[1] = 0x40;	/* set pef enabled, per-msg auth enabled */
 	if (enable != 0)
 		rqdata[1] |= 0x2; /* set always available if enable is set */
 	rqdata[2] = 0x44; 	/* set channel privilege limit to ADMIN */
@@ -870,7 +1034,7 @@ ipmi_set_channel_access(struct ipmi_intf * intf, uint8_t channel, uint8_t enable
 	/* SAVE TO CURRENT */
 	memset(rqdata, 0, 3);
 	rqdata[0] = channel & 0xf;
-	rqdata[1] = 0xa0;	/* set pef disabled, per-msg auth enabled */
+	rqdata[1] = 0x80;	/* set pef enabled, per-msg auth enabled */
 	if (enable != 0)
 		rqdata[1] |= 0x2; /* set always available if enable is set */
 	rqdata[2] = 0x84; 	/* set channel privilege limit to ADMIN */
@@ -1053,9 +1217,12 @@ static void ipmi_lan_set_usage(void)
 	lprintf(LOG_NOTICE, "  snmp <community string>        Set SNMP public community string");
 	lprintf(LOG_NOTICE, "  user                           Enable default user for this channel");
 	lprintf(LOG_NOTICE, "  access <on|off>                Enable or disable access to this channel");
-	lprintf(LOG_NOTICE, "  arp response <on|off>          Enable or disable BMC ARP responding");
+	lprintf(LOG_NOTICE, "  alert <on|off>                 Enable or disable PEF alerting for this channel");
+	lprintf(LOG_NOTICE, "  arp respond <on|off>           Enable or disable BMC ARP responding");
 	lprintf(LOG_NOTICE, "  arp generate <on|off>          Enable or disable BMC gratuitous ARP generation");
 	lprintf(LOG_NOTICE, "  arp interval <seconds>         Set gratuitous ARP generation interval");
+	lprintf(LOG_NOTICE, "  vlan id <off|<id>>             Disable or enable VLAN and set ID (1-4094)");
+	lprintf(LOG_NOTICE, "  vlan priority <priority>       Set vlan priority (0-7)");
 	lprintf(LOG_NOTICE, "  auth <level> <type,..>         Set channel authentication types");
 	lprintf(LOG_NOTICE, "    level  = CALLBACK, USER, OPERATOR, ADMIN");
 	lprintf(LOG_NOTICE, "    type   = NONE, MD2, MD5, PASSWORD, OEM");
@@ -1073,11 +1240,62 @@ static void ipmi_lan_set_usage(void)
 	lprintf(LOG_NOTICE, "    O = OEM\n");
 }
 
+static void
+ipmi_lan_set_vlan_usage(void)
+{
+	lprintf(LOG_NOTICE,
+		"lan set <channel> vlan id <id>\n"
+		"lan set <channel> vlan id off\n"
+		"lan set <channel> vlan priority <priority>\n");
+}
+
+static int
+ipmi_lan_set_vlan_id(struct ipmi_intf * intf,  uint8_t chan, char *string)
+{
+	uint8_t data[2];
+	int rc;
+
+	if (string == NULL) {
+		data[0] = 0;
+		data[1] = 0;
+	}
+	else {
+		int id = atoi(string);
+
+		if (id < 1 || id > 4094) {
+			lprintf(LOG_NOTICE, "vlan id must be between 1 and 4094.");
+			return -1;
+		}
+		else {
+			data[0] = (uint8_t)id;
+			data[1] = (uint8_t)(id >> 8) | 0x80;
+		}
+	}
+	rc = set_lan_param(intf, chan, IPMI_LANP_VLAN_ID, data, 2);
+	return rc;
+}
+
+static int
+ipmi_lan_set_vlan_priority(struct ipmi_intf * intf,  uint8_t chan, char *string)
+{
+	uint8_t data;
+	int rc;
+	int priority = atoi(string);
+
+	if (priority < 0 || priority > 7) {
+		lprintf(LOG_NOTICE, "vlan priority must be between 0 and 7.");
+		return -1;
+	}
+	data = (uint8_t)priority;
+	rc = set_lan_param(intf, chan, IPMI_LANP_VLAN_PRIORITY, &data, 1);
+	return rc;
+}
+
 static int
 ipmi_lan_set(struct ipmi_intf * intf, int argc, char ** argv)
 {
 	uint8_t data[32];
-	uint8_t chan, medium;
+	uint8_t chan;
 	int rc = 0;
 
 	if (argc < 2) {
@@ -1093,17 +1311,10 @@ ipmi_lan_set(struct ipmi_intf * intf, int argc, char ** argv)
 
 	chan = (uint8_t)strtol(argv[0], NULL, 0);
 
-	if (chan < 1 || chan > IPMI_CHANNEL_NUMBER_MAX) {
-		lprintf(LOG_ERR, "Invalid Channel %d", chan);
-		ipmi_lan_set_usage();
-		return -1;
-	}
-
 	/* find type of channel and only accept 802.3 LAN */
-	medium = ipmi_get_channel_medium(intf, chan);
-	if (medium != IPMI_CHANNEL_MEDIUM_LAN &&
-	    medium != IPMI_CHANNEL_MEDIUM_LAN_OTHER) {
+	if (is_lan_channel(intf, chan) == 0) {
 		lprintf(LOG_ERR, "Channel %d is not a LAN channel!", chan);
+		ipmi_lan_set_usage();
 		return -1;
 	}
 
@@ -1138,7 +1349,7 @@ ipmi_lan_set(struct ipmi_intf * intf, int argc, char ** argv)
 				"example: lan set 7 arp gratuitous off\n");
 		}
 		else if (strncmp(argv[2], "interval", 8) == 0) {
-			rc = lan_set_arp_interval(intf, chan, argv[3]);
+			rc = lan_set_arp_interval(intf, chan, (uint8_t *)argv[3]);
 		}
 		else if (strncmp(argv[2], "generate", 8) == 0) {
 			if (argc < 4)
@@ -1188,6 +1399,7 @@ ipmi_lan_set(struct ipmi_intf * intf, int argc, char ** argv)
 				"  static = static address (manually configured)\n"
 				"  dhcp   = address obtained by BMC running DHCP\n"
 				"  bios   = address loaded by BIOS or system software\n");
+			return 0;
 		}
 		else if (strncmp(argv[2], "none", 4) == 0)
 			data[0] = 0;
@@ -1211,7 +1423,7 @@ ipmi_lan_set(struct ipmi_intf * intf, int argc, char ** argv)
 	/* session password
 	 * not strictly a lan setting, but its used for lan connections */
 	else if (strncmp(argv[1], "password", 8) == 0) {
-		rc = ipmi_lan_set_password(intf, 1, argv[2]);
+		rc = ipmi_lan_set_password(intf, 1, (uint8_t *)argv[2]);
 	}
 	/* snmp community string */
 	else if (strncmp(argv[1], "snmp", 4) == 0) {
@@ -1288,7 +1500,44 @@ ipmi_lan_set(struct ipmi_intf * intf, int argc, char ** argv)
 			rc = set_lan_param(intf, chan, IPMI_LANP_BAK_GATEWAY_MAC, data, 6);
 		}
 	}
-
+	else if (strncasecmp(argv[1], "vlan", 4) == 0) {
+		if (argc < 4 || strncmp(argv[2], "help", 4) == 0) {
+			ipmi_lan_set_vlan_usage();
+		}
+		else if (strncasecmp(argv[2], "id", 2) == 0) {
+			if (strncasecmp(argv[3], "off", 3) == 0) {
+				ipmi_lan_set_vlan_id(intf, chan, NULL);
+			}
+			else {
+				ipmi_lan_set_vlan_id(intf, chan, argv[3]);
+			}
+		}
+		else if (strncasecmp(argv[2], "priority", 8) == 0) {
+			ipmi_lan_set_vlan_priority(intf, chan, argv[3]);
+		}
+		else {
+			ipmi_lan_set_vlan_usage();
+		}
+	}
+	/* set PEF alerting on or off */
+	else if (strncasecmp(argv[1], "alert", 5) == 0) {
+		if (argc < 3) {
+			lprintf(LOG_NOTICE, "LAN set alert must be 'on' or 'off'");
+		}
+		else if (strncasecmp(argv[2], "on", 2) == 0 ||
+			 strncasecmp(argv[2], "enable", 6) == 0) {
+			printf("Enabling PEF alerts for LAN channel %d\n", chan);
+			rc = ipmi_set_alert_enable(intf, chan, 1);
+		}
+		else if (strncasecmp(argv[2], "off", 3) == 0 ||
+			 strncasecmp(argv[2], "disable", 7) == 0) {
+			printf("Disabling PEF alerts for LAN channel %d\n", chan);
+			rc = ipmi_set_alert_enable(intf, chan, 0);
+		}
+		else {
+			lprintf(LOG_NOTICE, "LAN set alert must be 'on' or 'off'");
+		}
+	}
 	/* RMCP+ cipher suite privilege levels */
 	else if (strncmp(argv[1], "cipher_privs", 12) == 0)
 	{
@@ -1309,37 +1558,568 @@ ipmi_lan_set(struct ipmi_intf * intf, int argc, char ** argv)
 			rc = set_lan_param(intf, chan, IPMI_LANP_RMCP_PRIV_LEVELS, data, 9);
 		}
 	}
+	else {
+		ipmi_lan_set_usage();
+	}
 		
 	return rc;
 }
 
 
+static int
+is_alert_destination(struct ipmi_intf * intf, uint8_t channel, uint8_t alert)
+{
+	struct lan_param * p;
+
+	p = get_lan_param(intf, channel, IPMI_LANP_NUM_DEST);
+	if (p == NULL)
+		return 0;
+	if (p->data == NULL)
+		return 0;
+
+	if (alert <= (p->data[0] & 0xf))
+		return 1;
+	else
+		return 0;
+}
+
+static int
+ipmi_lan_alert_print(struct ipmi_intf * intf, uint8_t channel, uint8_t alert)
+{
+	struct lan_param * ptype, * paddr;
+	int isack = 0;
+
+	ptype = get_lan_param_select(intf, channel, IPMI_LANP_DEST_TYPE, alert);
+	paddr = get_lan_param_select(intf, channel, IPMI_LANP_DEST_ADDR, alert);
+	if (ptype == NULL || paddr == NULL)
+		return -1;
+	if (ptype->data == NULL || paddr->data == NULL)
+		return -1;
+
+	printf("%-24s: %d\n", "Alert Destination",
+	       ptype->data[0]);
+
+	if (ptype->data[1] & 0x80)
+		isack = 1;
+	printf("%-24s: %s\n", "Alert Acknowledge",
+	       isack ? "Acknowledged" : "Unacknowledged");
+
+	printf("%-24s: ", "Destination Type");
+	switch (ptype->data[1] & 0x7) {
+	case 0:
+		printf("PET Trap\n");
+		break;
+	case 6:
+		printf("OEM 1\n");
+		break;
+	case 7:
+		printf("OEM 2\n");
+		break;
+	default:
+		printf("Unknown\n");
+		break;
+	}
+
+	printf("%-24s: %d\n",
+	       isack ? "Acknowledge Timeout" : "Retry Interval",
+	       ptype->data[2]);
+
+	printf("%-24s: %d\n", "Number of Retries",
+	       ptype->data[3] & 0x7);
+
+	if ((paddr->data[1] & 0xf0) != 0) {
+		/* unknown address format */
+		printf("\n");
+		return 0;
+	}
+
+	printf("%-24s: %s\n", "Alert Gateway",
+	       (paddr->data[2] & 1) ? "Backup" : "Default");
+
+	printf("%-24s: %d.%d.%d.%d\n", "Alert IP Address",
+	       paddr->data[3], paddr->data[4], paddr->data[5], paddr->data[6]);
+
+	printf("%-24s: %02x:%02x:%02x:%02x:%02x:%02x\n", "Alert MAC Address",
+	       paddr->data[7], paddr->data[8], paddr->data[9],
+	       paddr->data[10], paddr->data[11], paddr->data[12]);
+
+	printf("\n");
+	return 0;
+}
+
+static int
+ipmi_lan_alert_print_all(struct ipmi_intf * intf, uint8_t channel)
+{
+	int j, ndest;
+	struct lan_param * p;
+
+	p = get_lan_param(intf, channel, IPMI_LANP_NUM_DEST);
+	if (p == NULL)
+		return -1;
+	if (p->data == NULL)
+		return -1;
+	ndest = p->data[0] & 0xf;
+
+	for (j=0; j<=ndest; j++) {
+		ipmi_lan_alert_print(intf, channel, j);
+	}
+
+	return 0;
+}
+
+static void
+ipmi_lan_alert_print_usage(void)
+{
+	lprintf(LOG_NOTICE, "\nusage: lan alert print [channel] [alert]\n");
+	lprintf(LOG_NOTICE, "Default will print all alerts for the first found LAN channel");
+}
+
+static void
+ipmi_lan_alert_set_usage(void)
+{
+	lprintf(LOG_NOTICE, "\nusage: lan alert set <channel> <alert> <command> <option>\n");
+	lprintf(LOG_NOTICE, "  ipaddr <x.x.x.x>               Set alert IP address");
+	lprintf(LOG_NOTICE, "  macaddr <x:x:x:x:x:x>          Set alert MAC address");
+	lprintf(LOG_NOTICE, "  gateway <default|backup>       Set channel gateway to use for alerts");
+	lprintf(LOG_NOTICE, "  ack <on|off>                   Set Alert Acknowledge on or off");
+	lprintf(LOG_NOTICE, "  type <pet|oem1|oem2>           Set destination type as PET or OEM");
+	lprintf(LOG_NOTICE, "  time <seconds>                 Set ack timeout or unack retry interval");
+	lprintf(LOG_NOTICE, "  retry <number>                 Set number of alert retries");
+	lprintf(LOG_NOTICE, "");
+}
+
+static int
+ipmi_lan_alert_set(struct ipmi_intf * intf, uint8_t chan, uint8_t alert,
+		   int argc, char ** argv)
+{
+	struct lan_param * p;
+	uint8_t data[32], temp[32];
+	int rc = 0;
+
+	if (argc < 2) {
+		ipmi_lan_alert_set_usage();
+		return 0;
+	}
+
+	if (strncmp(argv[0], "help", 4) == 0 ||
+	    strncmp(argv[1], "help", 4) == 0) {
+		ipmi_lan_alert_set_usage();
+		return 0;
+	}
+
+	memset(data, 0, sizeof(data));
+	memset(temp, 0, sizeof(temp));
+
+	/* alert destination ip address */
+	if (strncasecmp(argv[0], "ipaddr", 6) == 0 &&
+	    (get_cmdline_ipaddr(argv[1], temp) == 0)) {
+		/* get current parameter */
+		p = get_lan_param_select(intf, chan, IPMI_LANP_DEST_ADDR, alert);
+		memcpy(data, p->data, p->data_len);
+		/* set new ipaddr */
+		memcpy(data+3, temp, 4);
+		printf("Setting LAN Alert %d IP Address to %d.%d.%d.%d\n", alert,
+		       data[3], data[4], data[5], data[6]);
+		rc = set_lan_param_nowait(intf, chan, IPMI_LANP_DEST_ADDR, data, p->data_len);
+	}
+	/* alert destination mac address */
+	else if (strncasecmp(argv[0], "macaddr", 7) == 0 &&
+		 (get_cmdline_macaddr(argv[1], temp) == 0)) {
+		/* get current parameter */
+		p = get_lan_param_select(intf, chan, IPMI_LANP_DEST_ADDR, alert);
+		memcpy(data, p->data, p->data_len);
+		/* set new macaddr */
+		memcpy(data+7, temp, 6);
+		printf("Setting LAN Alert %d MAC Address to "
+		       "%02x:%02x:%02x:%02x:%02x:%02x\n", alert,
+		       data[7], data[8], data[9], data[10], data[11], data[12]);
+		rc = set_lan_param_nowait(intf, chan, IPMI_LANP_DEST_ADDR, data, p->data_len);
+	}
+	/* alert destination gateway selector */
+	else if (strncasecmp(argv[0], "gateway", 7) == 0) {
+		/* get current parameter */
+		p = get_lan_param_select(intf, chan, IPMI_LANP_DEST_ADDR, alert);
+		memcpy(data, p->data, p->data_len);
+
+		if (strncasecmp(argv[1], "def", 3) == 0 ||
+		    strncasecmp(argv[1], "default", 7) == 0) {
+			printf("Setting LAN Alert %d to use Default Gateway\n", alert);
+			data[2] = 0;
+		}
+		else if (strncasecmp(argv[1], "bak", 3) == 0 ||
+			 strncasecmp(argv[1], "backup", 6) == 0) {
+			printf("Setting LAN Alert %d to use Backup Gateway\n", alert);
+			data[2] = 1;
+		}
+		else {
+			ipmi_lan_alert_set_usage();
+			return -1;
+		}
+
+		rc = set_lan_param_nowait(intf, chan, IPMI_LANP_DEST_ADDR, data, p->data_len);
+	}
+	/* alert acknowledgement */
+	else if (strncasecmp(argv[0], "ack", 3) == 0) {
+		/* get current parameter */
+		p = get_lan_param_select(intf, chan, IPMI_LANP_DEST_TYPE, alert);
+		memcpy(data, p->data, p->data_len);
+
+		if (strncasecmp(argv[1], "on", 2) == 0 ||
+		    strncasecmp(argv[1], "yes", 3) == 0) {
+			printf("Setting LAN Alert %d to Acknowledged\n", alert);
+			data[1] |= 0x80;
+		}
+		else if (strncasecmp(argv[1], "off", 3) == 0 ||
+			 strncasecmp(argv[1], "no", 2) == 0) {
+			printf("Setting LAN Alert %d to Unacknowledged\n", alert);
+			data[1] &= ~0x80;
+		}
+		else {
+			ipmi_lan_alert_set_usage();
+			return -1;
+		}
+		rc = set_lan_param_nowait(intf, chan, IPMI_LANP_DEST_TYPE, data, p->data_len);
+	}
+	/* alert destination type */
+	else if (strncasecmp(argv[0], "type", 4) == 0) {
+		/* get current parameter */
+		p = get_lan_param_select(intf, chan, IPMI_LANP_DEST_TYPE, alert);
+		memcpy(data, p->data, p->data_len);
+
+		if (strncasecmp(argv[1], "pet", 3) == 0) {
+			printf("Setting LAN Alert %d destination to PET Trap\n", alert);
+			data[1] &= ~0x07;
+		}
+		else if (strncasecmp(argv[1], "oem1", 4) == 0) {
+			printf("Setting LAN Alert %d destination to OEM 1\n", alert);
+			data[1] &= ~0x07;
+			data[1] |= 0x06;
+		}
+		else if (strncasecmp(argv[1], "oem2", 4) == 0) {
+			printf("Setting LAN Alert %d destination to OEM 2\n", alert);
+			data[1] |= 0x07;
+		}
+		else {
+			ipmi_lan_alert_set_usage();
+			return -1;
+		}
+		rc = set_lan_param_nowait(intf, chan, IPMI_LANP_DEST_TYPE, data, p->data_len);
+	}
+	/* alert acknowledge timeout or retry interval */
+	else if (strncasecmp(argv[0], "time", 4) == 0) {
+		/* get current parameter */
+		p = get_lan_param_select(intf, chan, IPMI_LANP_DEST_TYPE, alert);
+		memcpy(data, p->data, p->data_len);
+
+		data[2] = (uint8_t)strtol(argv[1], NULL, 0);
+		printf("Setting LAN Alert %d timeout/retry to %d seconds\n", alert, data[2]);
+		rc = set_lan_param_nowait(intf, chan, IPMI_LANP_DEST_TYPE, data, p->data_len);
+	}
+	/* number of retries */
+	else if (strncasecmp(argv[0], "retry", 5) == 0) {
+		/* get current parameter */
+		p = get_lan_param_select(intf, chan, IPMI_LANP_DEST_TYPE, alert);
+		memcpy(data, p->data, p->data_len);
+
+		data[3] = ((uint8_t)strtol(argv[1], NULL, 0)) & 0x7;
+		printf("Setting LAN Alert %d number of retries to %d\n", alert, data[3]);
+		rc = set_lan_param_nowait(intf, chan, IPMI_LANP_DEST_TYPE, data, p->data_len);
+	}
+	else {
+		ipmi_lan_alert_set_usage();
+		return -1;
+	}
+
+	return rc;
+}
+
+static int
+ipmi_lan_alert(struct ipmi_intf * intf, int argc, char ** argv)
+{
+	uint8_t alert;
+	uint8_t channel = 1;
+
+	if (argc < 1 ||
+	    strncasecmp(argv[0], "help", 4) == 0) {
+		ipmi_lan_alert_print_usage();
+		ipmi_lan_alert_set_usage();
+		return 0;
+	}
+
+	/* alert print [channel] [alert] */
+	if (strncasecmp(argv[0], "print", 5) == 0) {
+		if (argc < 2) {
+			channel = find_lan_channel(intf, 1);
+			if (is_lan_channel(intf, channel) == 0) {
+				lprintf(LOG_ERR, "Channel %d is not a LAN channel", channel);
+				return -1;
+			}
+			return ipmi_lan_alert_print_all(intf, channel);
+		}
+
+		if (strncasecmp(argv[1], "help", 4) == 0) {
+			ipmi_lan_alert_print_usage();
+			return 0;
+		}
+
+		channel = (uint8_t)strtol(argv[1], NULL, 0);
+		if (is_lan_channel(intf, channel) == 0) {
+			lprintf(LOG_ERR, "Channel %d is not a LAN channel", channel);
+			return -1;
+		}
+
+		if (argc < 3)
+			return ipmi_lan_alert_print_all(intf, channel);
+
+		alert = (uint8_t)strtol(argv[2], NULL, 0);
+		if (is_alert_destination(intf, channel, alert) == 0) {
+			lprintf(LOG_ERR, "Alert %d is not a valid destination", alert);
+			return -1;
+		}
+		return ipmi_lan_alert_print(intf, channel, alert);
+	}
+
+	/* alert set <channel> <alert> [option] */
+	if (strncasecmp(argv[0], "set", 3) == 0) {
+		if (argc < 5 || strncasecmp(argv[1], "help", 4) == 0) {
+			ipmi_lan_alert_set_usage();
+			return 0;
+		}
+
+		channel = (uint8_t)strtol(argv[1], NULL, 0);
+		if (is_lan_channel(intf, channel) == 0) {
+			lprintf(LOG_ERR, "Channel %d is not a LAN channel", channel);
+			return -1;
+		}
+
+		alert = (uint8_t)strtol(argv[2], NULL, 0);
+		if (is_alert_destination(intf, channel, alert) == 0) {
+			lprintf(LOG_ERR, "Alert %d is not a valid destination", alert);
+			return -1;
+		}
+
+		return ipmi_lan_alert_set(intf, channel, alert, argc-3, &(argv[3]));
+	}
+
+	return 0;
+}
+
+
+static int
+ipmi_lan_stats_get(struct ipmi_intf * intf, uint8_t chan)
+{
+	uint8_t medium;
+	int rc = 0;
+	struct ipmi_rs * rsp;
+	struct ipmi_rq req;
+	uint8_t msg_data[2];
+	uint16_t statsTemp;
+
+	if (chan < 1 || chan > IPMI_CHANNEL_NUMBER_MAX) {
+		lprintf(LOG_ERR, "Invalid Channel %d", chan);
+		return -1;
+	}
+
+	/* find type of channel and only accept 802.3 LAN */
+	medium = ipmi_get_channel_medium(intf, chan);
+	if (medium != IPMI_CHANNEL_MEDIUM_LAN &&
+	    medium != IPMI_CHANNEL_MEDIUM_LAN_OTHER) {
+		lprintf(LOG_ERR, "Channel %d (%s) is not a LAN channel",
+			chan, val2str(medium, ipmi_channel_medium_vals), medium);
+		return -1;
+	}
+
+	/* From here, we are ready to get the stats */
+
+	msg_data[0] = chan;
+	msg_data[1] = 0;   /* Don't clear */
+
+	memset(&req, 0, sizeof(req));
+	req.msg.netfn    = IPMI_NETFN_TRANSPORT;
+	req.msg.cmd      = IPMI_LAN_GET_STAT;
+	req.msg.data     = msg_data;
+	req.msg.data_len = 2;
+
+	rsp = intf->sendrecv(intf, &req);
+	if (rsp == NULL) {
+		lprintf(LOG_INFO, "Get LAN Stats command failed");
+		return 0;
+	}
+
+	if (rsp->ccode > 0) {
+		lprintf(LOG_INFO, "Get LAN Stats command failed: %s",
+			val2str(rsp->ccode, completion_code_vals));
+		return 0;
+	}
+      
+	if (verbose > 1)
+	{
+		uint8_t counter;
+		printf("--- Rx Stats ---\n"); 
+		for(counter=0;counter<18;counter+=2)
+		{
+			printf("%02X", *(rsp->data + counter)); 
+			printf(" %02X - ", *(rsp->data + counter+1)); 
+		} 
+		printf("\n"); 
+	}       
+
+	statsTemp = ((*(rsp->data + 0)) << 8) | (*(rsp->data + 1));
+	printf("IP Rx Packet              : %d\n", statsTemp);
+
+	statsTemp = ((*(rsp->data + 2)) << 8) | (*(rsp->data + 3));
+	printf("IP Rx Header Errors       : %u\n", statsTemp);
+
+	statsTemp = ((*(rsp->data + 4)) << 8) | (*(rsp->data + 5));
+	printf("IP Rx Address Errors      : %u\n", statsTemp);
+
+	statsTemp = ((*(rsp->data + 6)) << 8) | (*(rsp->data + 7));
+	printf("IP Rx Fragmented          : %u\n", statsTemp);
+
+	statsTemp = ((*(rsp->data + 8)) << 8) | (*(rsp->data + 9));
+	printf("IP Tx Packet              : %u\n", statsTemp);
+
+	statsTemp = ((*(rsp->data +10)) << 8) | (*(rsp->data +11));
+	printf("UDP Rx Packet             : %u\n", statsTemp);
+
+	statsTemp = ((*(rsp->data + 12)) << 8) | (*(rsp->data + 13));
+	printf("RMCP Rx Valid             : %u\n", statsTemp);
+
+	statsTemp = ((*(rsp->data + 14)) << 8) | (*(rsp->data + 15));
+	printf("UDP Proxy Packet Received : %u\n", statsTemp);
+
+	statsTemp = ((*(rsp->data + 16)) << 8) | (*(rsp->data + 17));
+	printf("UDP Proxy Packet Dropped  : %u\n", statsTemp);
+
+	return rc;
+}
+
+
+static int
+ipmi_lan_stats_clear(struct ipmi_intf * intf, uint8_t chan)
+{
+	uint8_t medium;
+	int rc = 0;
+	struct ipmi_rs * rsp;
+	struct ipmi_rq req;
+	uint8_t msg_data[2];
+
+	if (chan < 1 || chan > IPMI_CHANNEL_NUMBER_MAX) {
+		lprintf(LOG_ERR, "Invalid Channel %d", chan);
+		return -1;
+	}
+
+	/* find type of channel and only accept 802.3 LAN */
+	medium = ipmi_get_channel_medium(intf, chan);
+	if (medium != IPMI_CHANNEL_MEDIUM_LAN &&
+	    medium != IPMI_CHANNEL_MEDIUM_LAN_OTHER) {
+		lprintf(LOG_ERR, "Channel %d (%s) is not a LAN channel",
+			chan, val2str(medium, ipmi_channel_medium_vals), medium);
+		return -1;
+	}
+
+	/* From here, we are ready to get the stats */
+	msg_data[0] = chan;
+	msg_data[1] = 1;   /* Clear */
+
+	memset(&req, 0, sizeof(req));
+	req.msg.netfn    = IPMI_NETFN_TRANSPORT;
+	req.msg.cmd      = IPMI_LAN_GET_STAT;
+	req.msg.data     = msg_data;
+	req.msg.data_len = 2;
+
+	rsp = intf->sendrecv(intf, &req);
+	if (rsp == NULL) {
+		lprintf(LOG_INFO, "Get LAN Stats command failed");
+		return 0;
+	}
+
+	if (rsp->ccode > 0) {
+		lprintf(LOG_INFO, "Get LAN Stats command failed: %s",
+			val2str(rsp->ccode, completion_code_vals));
+		return 0;
+	}
+
+	return rc;
+}
+
+
+/*
+ * print_lan_usage
+ */
+static void
+print_lan_usage(void)
+{
+	lprintf(LOG_NOTICE, "LAN Commands:");
+	lprintf(LOG_NOTICE, "		   print [<channel number>]");
+	lprintf(LOG_NOTICE, "		   set [<channel number>]");
+	lprintf(LOG_NOTICE, "		   alert [<channel number>]");
+	lprintf(LOG_NOTICE, "		   stats get [<channel number>]");
+	lprintf(LOG_NOTICE, "		   stats clear [<channel number>]");
+}
+
+
 int
 ipmi_lanp_main(struct ipmi_intf * intf, int argc, char ** argv)
 {
 	int rc = 0;
+   uint8_t chan = 0x0e;
 
 	if (argc == 0 || (strncmp(argv[0], "help", 4) == 0)) {
-		lprintf(LOG_NOTICE, "LAN Commands:  print, set");
-		return -1;
+		print_lan_usage();
+      return -1;
 	}
 
-
-	if ((strncmp(argv[0], "printconf", 9) == 0) ||
-	    (strncmp(argv[0], "print", 5) == 0)) {
-		uint8_t chan = 7;
+	if (
+         (strncmp(argv[0], "printconf", 9) == 0) 
+         ||
+	      (strncmp(argv[0], "print", 5) == 0) 
+      )
+   {
 		if (argc > 1)
 			chan = (uint8_t)strtol(argv[1], NULL, 0);
 		if (chan < 1 || chan > IPMI_CHANNEL_NUMBER_MAX)
-			lprintf(LOG_NOTICE, "usage: lan print <channel>");
+      {
+         lprintf(LOG_ERR, "Invalid channel: %d", chan);
+			return -1;
+       }
 		else
 			rc = ipmi_lan_print(intf, chan);
 	}
 	else if (strncmp(argv[0], "set", 3) == 0)
-		rc = ipmi_lan_set(intf, argc-1, &(argv[1]));
-	else
-		lprintf(LOG_NOTICE, "Invalid LAN command: %s", argv[0]);
-
+	{
+   	rc = ipmi_lan_set(intf, argc-1, &(argv[1]));
+	}
+	else if (strncmp(argv[0], "alert", 5) == 0)
+	{
+   	rc = ipmi_lan_alert(intf, argc-1, &(argv[1]));
+   }
+   else if (strncmp(argv[0], "stats", 5) == 0)
+   {
+      if(argc == 1)
+      {
+   		print_lan_usage();
+			return -1;
+      }
+      else if (argc > 2)
+	   	chan = (uint8_t)strtol(argv[2], NULL, 0);
+         
+   	if (chan < 2 || chan > IPMI_CHANNEL_NUMBER_MAX)
+      {
+         lprintf(LOG_ERR, "Invalid channel: %d", chan);
+			return -1;
+      }
+      else if (strncmp(argv[1], "get", 3) == 0)
+	   	rc = ipmi_lan_stats_get(intf, chan);
+      else if (strncmp(argv[1], "clear", 5) == 0)
+	   	rc = ipmi_lan_stats_clear(intf, chan);
+      else
+   		print_lan_usage();
+	}
+   else
+	{
+   	lprintf(LOG_NOTICE, "Invalid LAN command: %s", argv[0]);
+   }
 	return rc;
 }
 
