@@ -28,10 +28,6 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #include <stdlib.h>
@@ -42,11 +38,17 @@
 #include <sys/stat.h>
 #include <sys/select.h>
 #include <sys/time.h>
+#include <time.h>
 #include <signal.h>
 #include <unistd.h>
 
+#include <config.h>
 
-#include <termios.h>
+#if defined(HAVE_TERMIOS_H)
+# include <termios.h>
+#elif defined (HAVE_SYS_TERMIOS_H)
+# include <sys/termios.h>
+#endif        
 
 #include <ipmitool/helper.h>
 #include <ipmitool/log.h>
@@ -67,8 +69,6 @@
 #define SOL_PARAMETER_SOL_PAYLOAD_CHANNEL       0x07
 #define SOL_PARAMETER_SOL_PAYLOAD_PORT          0x08
 
-#define SOL_ESCAPE_CHARACTER                    '~'
-
 const struct valstr sol_parameter_vals[] = {
 	{ SOL_PARAMETER_SET_IN_PROGRESS,           "Set In Progress (0)" },
 	{ SOL_PARAMETER_SOL_ENABLE,                "Enable (1)" },
@@ -83,11 +83,13 @@ const struct valstr sol_parameter_vals[] = {
 };
 
 
+static struct timeval _start_keepalive;
 static struct termios _saved_tio;
 static int            _in_raw_mode = 0;
 
 extern int verbose;
 
+static uint8_t G_u8ActiveSOL = 0;
 
 /*
  * ipmi_get_sol_info
@@ -102,6 +104,7 @@ ipmi_get_sol_info(
 	struct ipmi_rq req;
 	uint8_t data[4];	
 
+	memset(&req, 0, sizeof(req)); 
 	req.msg.netfn    = IPMI_NETFN_TRANSPORT;
 	req.msg.cmd      = IMPI_GET_SOL_CONFIG_PARAMETERS;
 	req.msg.data_len = 4;
@@ -547,6 +550,7 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 	uint8_t    data[4];	 
 	int              bGuarded = 1; /* Use set-in-progress indicator? */
 
+   memset(&req, 0, sizeof(req));
 	req.msg.netfn    = IPMI_NETFN_TRANSPORT;           /* 0x0c */ 
 	req.msg.cmd      = IMPI_SET_SOL_CONFIG_PARAMETERS; /* 0x21 */
 	req.msg.data     = data;
@@ -769,7 +773,7 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 
 		req.msg.data_len = 4;
 		data[1] = SOL_PARAMETER_SOL_RETRY;
-		data[2] = (uint8_t)strtol(value, NULL, 0) & 0x03;
+		data[2] = (uint8_t)strtol(value, NULL, 0) & 0x07;
 
 		/* We need other values to complete the request */
 		if (ipmi_get_sol_info(intf, channel, &params))
@@ -1046,7 +1050,7 @@ suspendSelf(int bRestoreTty)
  * Send some useful documentation to the user
  */
 static void
-printSolEscapeSequences(void)
+printSolEscapeSequences(struct ipmi_intf * intf)
 {
 	printf(
 		   "%c?\r\n\
@@ -1058,14 +1062,14 @@ printSolEscapeSequences(void)
 	%c?  - this message\r\n\
 	%c%c  - send the escape character by typing it twice\r\n\
 	(Note that escapes are only recognized immediately after newline.)\r\n",
-		   SOL_ESCAPE_CHARACTER,
-		   SOL_ESCAPE_CHARACTER,
-		   SOL_ESCAPE_CHARACTER,
-		   SOL_ESCAPE_CHARACTER,
-		   SOL_ESCAPE_CHARACTER,
-		   SOL_ESCAPE_CHARACTER,
-		   SOL_ESCAPE_CHARACTER,
-		   SOL_ESCAPE_CHARACTER);
+		   intf->session->sol_escape_char,
+		   intf->session->sol_escape_char,
+		   intf->session->sol_escape_char,
+		   intf->session->sol_escape_char,
+		   intf->session->sol_escape_char,
+		   intf->session->sol_escape_char,
+		   intf->session->sol_escape_char,
+		   intf->session->sol_escape_char);
 }
 
 
@@ -1100,6 +1104,7 @@ ipmi_sol_deactivate(struct ipmi_intf * intf)
 	struct ipmi_rq   req;
 	uint8_t    data[6];	 
 
+   memset(&req, 0, sizeof(req));
 	req.msg.netfn    = IPMI_NETFN_APP;
 	req.msg.cmd      = IPMI_DEACTIVATE_PAYLOAD;
 	req.msg.data_len = 6;
@@ -1155,7 +1160,7 @@ static int
 processSolUserInput(
 					struct ipmi_intf * intf,
 					uint8_t    * input,
-					uint16_t     buffer_length)
+					uint16_t   buffer_length)
 {
 	static int escape_pending = 0;
 	static int last_was_cr    = 1;
@@ -1183,38 +1188,44 @@ processSolUserInput(
 			 */
 			switch (ch) {
 			case '.':
-				printf("%c. [terminated ipmitool]\r\n", SOL_ESCAPE_CHARACTER);
+				printf("%c. [terminated ipmitool]\r\n",
+				       intf->session->sol_escape_char);
 				retval = 1;
 				break;
+
 			case 'Z' - 64:
-				printf("%c^Z [suspend ipmitool]\r\n", SOL_ESCAPE_CHARACTER);
+				printf("%c^Z [suspend ipmitool]\r\n",
+				       intf->session->sol_escape_char);
 				suspendSelf(1); /* Restore tty back to raw */
 				continue;
 
 			case 'X' - 64:
-				printf("%c^Z [suspend ipmitool]\r\n", SOL_ESCAPE_CHARACTER);
+				printf("%c^Z [suspend ipmitool]\r\n",
+				       intf->session->sol_escape_char);
 				suspendSelf(0); /* Don't restore to raw mode */
 				continue;
 
 			case 'B':
-				printf("%cb [send break]\r\n", SOL_ESCAPE_CHARACTER);
+				printf("%cb [send break]\r\n",
+				       intf->session->sol_escape_char);
 				sendBreak(intf);
 				continue;
 
 			case '?':
-				printSolEscapeSequences();
+				printSolEscapeSequences(intf);
 				continue;
+
 			default:
-				if (ch != SOL_ESCAPE_CHARACTER)
+				if (ch != intf->session->sol_escape_char)
 					v2_payload.payload.sol_packet.data[length++] =
-						SOL_ESCAPE_CHARACTER;
+						intf->session->sol_escape_char;
 				v2_payload.payload.sol_packet.data[length++] = ch;
 			}
 		}
 
 		else
 		{
-			if (last_was_cr && (ch == SOL_ESCAPE_CHARACTER)) {
+			if (last_was_cr && (ch == intf->session->sol_escape_char)) {
 				escape_pending = 1;
 				continue;
 			}
@@ -1237,28 +1248,57 @@ processSolUserInput(
 	 */
 	if (length)
 	{
-		struct ipmi_rs * rsp;
+		struct ipmi_rs * rsp = NULL;
+		int try = 0;
+
+		while (try < intf->session->retry) {
 		
-		v2_payload.payload.sol_packet.character_count = length;
-		rsp = intf->send_sol(intf, &v2_payload);
+			v2_payload.payload.sol_packet.character_count = length;
+			rsp = intf->send_sol(intf, &v2_payload);
+
+			if (rsp)
+			{
+				break;
+			}
+
+			usleep(5000);
+			try++;
+		}
 
 		if (! rsp)
 		{
-			lprintf(LOG_ERR, "Error sending SOL data");
+			lprintf(LOG_ERR, "Error sending SOL data: FAIL");
 			retval = -1;
 		}
 
 		/* If the sequence number is set we know we have new data */
-		else if ((rsp->session.authtype == IPMI_SESSION_AUTHTYPE_RMCP_PLUS) &&
-				 (rsp->session.payloadtype == IPMI_PAYLOAD_TYPE_SOL)        &&
-		 		 (rsp->payload.sol_packet.packet_sequence_number))
-			output(rsp);
+		if (retval == 0)
+			if ((rsp->session.authtype == IPMI_SESSION_AUTHTYPE_RMCP_PLUS) &&
+			    (rsp->session.payloadtype == IPMI_PAYLOAD_TYPE_SOL)        &&
+			    (rsp->payload.sol_packet.packet_sequence_number))
+				output(rsp);
 	}
 
 	return retval;
 }
 
 
+static int
+ipmi_sol_keepalive(struct ipmi_intf * intf)
+{
+	struct timeval end;
+	int ret = 0;
+
+	gettimeofday(&end, 0);
+
+	if (end.tv_sec - _start_keepalive.tv_sec > SOL_KEEPALIVE_TIMEOUT) {
+		ret = intf->keepalive(intf);
+		gettimeofday(&_start_keepalive, 0);
+	}
+
+	return ret;
+}
+
 
 /*
  * ipmi_sol_red_pill
@@ -1275,6 +1315,7 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 	struct timeval tv;
 	int    retval;
 	int    buffer_size = intf->session->sol_data.max_inbound_payload_size;
+	int    keepAliveRet = 0;
 
 	buffer = (char*)malloc(buffer_size);
 	if (buffer == NULL) {
@@ -1282,6 +1323,9 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 		return -1;
 	}
 
+	/* Initialize keepalive start time */
+	gettimeofday(&_start_keepalive, 0);	
+
 	enter_raw_mode();
 
 	while (! bShouldExit)
@@ -1290,6 +1334,15 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 		FD_SET(0, &read_fds);
 		FD_SET(intf->fd, &read_fds);
 
+		/* Send periodic keepalive packet */
+		keepAliveRet = ipmi_sol_keepalive(intf);
+		if (keepAliveRet != 0)
+		{
+			/* no response to keepalive message */
+			bShouldExit = 1;
+			continue;
+		}
+
 		/* Wait up to half a second */
 		tv.tv_sec =  0;
 		tv.tv_usec = 500000;
@@ -1318,7 +1371,7 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 				
 				if (numRead > 0)
 				{
-					int rc = processSolUserInput(intf, buffer, numRead);
+					int rc = processSolUserInput(intf, (uint8_t *)buffer, numRead);
 					
 					if (rc)
 					{
@@ -1363,6 +1416,14 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 
 	leave_raw_mode();
 
+	if (keepAliveRet != 0)
+	{
+		lprintf(LOG_ERR, "Error: No response to keepalive - Terminating session");
+		/* attempt to clean up anyway */
+		ipmi_sol_deactivate(intf);
+		exit(1);
+	}
+
 	if (bBmcClosedSession)
 	{
 		lprintf(LOG_ERR, "SOL session closed by BMC");
@@ -1381,7 +1442,7 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
  * impi_sol_activate
  */
 static int
-ipmi_sol_activate(struct ipmi_intf * intf)
+ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
 {
 	struct ipmi_rs * rsp;
 	struct ipmi_rq   req;
@@ -1410,6 +1471,7 @@ ipmi_sol_activate(struct ipmi_intf * intf)
 	intf->session->sol_data.sol_input_handler = output;
 
 
+	memset(&req, 0, sizeof(req));
 	req.msg.netfn    = IPMI_NETFN_APP;
 	req.msg.cmd      = IPMI_ACTIVATE_PAYLOAD;
 	req.msg.data_len = 6;
@@ -1422,12 +1484,18 @@ ipmi_sol_activate(struct ipmi_intf * intf)
 	data[2]  = bSolEncryption?     0x80 : 0;
 	data[2] |= bSolAuthentication? 0x40 : 0;
 	data[2] |= IPMI_SOL_SERIAL_ALERT_MASK_DEFERRED;
-	data[2] |= IPMI_SOL_BMC_ASSERTS_CTS_MASK_FALSE;
+
+	if (ipmi_oem_active(intf, "intelplus")) {
+		data[2] |= IPMI_SOL_BMC_ASSERTS_CTS_MASK_TRUE;
+	} else {
+		data[2] |= IPMI_SOL_BMC_ASSERTS_CTS_MASK_FALSE;
+	}
 
 	data[3] = 0x00; /* reserved */
 	data[4] = 0x00; /* reserved */
 	data[5] = 0x00; /* reserved */
-	
+
+	 G_u8ActiveSOL = 1; 
 	rsp = intf->sendrecv(intf, &req);
 
 	if (NULL != rsp) {
@@ -1497,16 +1565,32 @@ ipmi_sol_activate(struct ipmi_intf * intf)
 	intf->session->timeout = 3;
 
 
+	/* NOTE: the spec does allow for SOL traffic to be sent on
+	 * a different port.  we do not yet support that feature. */
 	if (intf->session->sol_data.port != intf->session->port)
 	{
-		lprintf(LOG_ERR, "Error: BMC requests SOL session on different port");
-		return -1;
+		/* try byteswapping port in case BMC sent it incorrectly */
+		uint16_t portswap = BSWAP_16(intf->session->sol_data.port);
+
+		if (portswap == intf->session->port) {
+			intf->session->sol_data.port = portswap;
+		}
+		else {
+			lprintf(LOG_ERR, "Error: BMC requests SOL session on different port");
+			return -1;
+		}
 	}
 	
 
 	printf("[SOL Session operational.  Use %c? for help]\r\n",
-		   SOL_ESCAPE_CHARACTER);
+	       intf->session->sol_escape_char);
 
+	if(looptest == 1)
+	{
+		ipmi_sol_deactivate(intf);
+		usleep(interval*1000);
+		return 0;
+	}	
 
 	/*
 	 * At this point we are good to go with our SOL session.  We
@@ -1535,6 +1619,7 @@ print_sol_usage(void)
 	lprintf(LOG_NOTICE, "              set <parameter> <value> [channel]");
 	lprintf(LOG_NOTICE, "              activate");
 	lprintf(LOG_NOTICE, "              deactivate");
+	lprintf(LOG_NOTICE, "              looptest [<loop times>] [<loop interval(in ms)>]");
 }
 
 
@@ -1626,7 +1711,7 @@ ipmi_sol_main(struct ipmi_intf * intf, int argc, char ** argv)
 	 * Activate
 	 */
  	else if (!strncmp(argv[0], "activate", 8))
-		retval = ipmi_sol_activate(intf);
+		retval = ipmi_sol_activate(intf, 0, 0);
 
 	/*
 	 * Dectivate
@@ -1635,6 +1720,43 @@ ipmi_sol_main(struct ipmi_intf * intf, int argc, char ** argv)
 		retval = ipmi_sol_deactivate(intf);
 	
 
+	/*
+	 * SOL loop test: Activate and then Dectivate
+	 */
+	else if (!strncmp(argv[0], "looptest", 8))
+	{
+		int cnt = 200;
+		int interval = 100; /* Unit is: ms */
+
+		if(argc > 3)
+		{
+			print_sol_usage();	
+			return -1;
+		}
+		if (argc != 1) /* at least 2 */
+		{
+			cnt = strtol(argv[1], NULL, 10);
+			if(cnt <= 0) cnt = 200;
+		}
+		if (argc == 3)
+		{
+			interval = strtol(argv[2], NULL, 10);
+			if(interval < 0) interval = 0;
+		}
+		
+		while(cnt > 0)
+		{
+			printf("remain loop test counter: %d\n", cnt);
+			retval = ipmi_sol_activate(intf, 1, interval);
+			if(retval)
+			{
+				printf("SOL looptest failed: %d\n", retval);
+				break;
+			}
+			cnt -= 1;
+		}
+	}
+
 	else
 	{
 		print_sol_usage();
