@@ -28,10 +28,6 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #include <stdlib.h>
@@ -142,6 +138,8 @@ ipmi_get_user_name(
 	struct ipmi_rq	       req;
 	uint8_t	       msg_data[1];
 
+	memset(user_name, 0, 17);
+
 	memset(&req, 0, sizeof(req));
 	req.msg.netfn    = IPMI_NETFN_APP;	   /* 0x06 */
 	req.msg.cmd	     = IPMI_GET_USER_NAME; /* 0x45 */
@@ -158,6 +156,8 @@ ipmi_get_user_name(
 		return -1;
 	}
 	if (rsp->ccode > 0) {
+		if (rsp->ccode == 0xcc)
+			return 0;
 		lprintf(LOG_ERR, "Get User Name command failed (user %d): %s",
 			user_id, val2str(rsp->ccode, completion_code_vals));
 		return -1;
@@ -326,7 +326,7 @@ ipmi_user_set_username(
 	/* The channel number will remain constant throughout this function */
 	msg_data[0] = user_id;
 	memset(msg_data + 1, 0, 16);
-	strcpy(msg_data + 1, name);
+	strcpy((char *)(msg_data + 1), name);
 
 	rsp = intf->sendrecv(intf, &req);
 
@@ -345,6 +345,47 @@ ipmi_user_set_username(
 }
 	
 
+static int
+ipmi_user_set_userpriv(
+		       struct ipmi_intf * intf,
+             uint8_t	  channel,
+		       uint8_t	  user_id,
+		       const unsigned char	privLevel)
+{
+	struct ipmi_rs	     * rsp;
+	struct ipmi_rq	       req;
+	uint8_t	       msg_data[17];
+
+	memset(&req, 0, sizeof(req));
+	req.msg.netfn    = IPMI_NETFN_APP;	     /* 0x06 */
+	req.msg.cmd	     = IPMI_SET_USER_ACCESS;   /* 0x43 */
+	req.msg.data     = msg_data;
+	req.msg.data_len = 3;
+
+
+	/* The channel number will remain constant throughout this function */
+	msg_data[0] = (channel   & 0x0f); /* Do not change any bytes */
+	msg_data[1] = (user_id   & 0x3f);
+	msg_data[2] = (privLevel & 0x0f);
+
+	rsp = intf->sendrecv(intf, &req);
+
+	if (rsp == NULL) 
+   {
+		lprintf(LOG_ERR, "Set Privilege Level command failed (user %d)",
+			user_id);
+		return -1;
+	}
+	if (rsp->ccode > 0) 
+   {
+		lprintf(LOG_ERR, "Set Privilege Level command failed (user %d)",
+			user_id);
+		return -1;
+	}
+
+	return 0;
+}
+
 
 /*
  * ipmi_user_set_password
@@ -363,11 +404,11 @@ ipmi_user_set_password(
 {
 	struct ipmi_rs	     * rsp;
 	struct ipmi_rq	       req;
-	char	             * msg_data;
+	uint8_t	             * msg_data;
 
 	int password_length = (is_twenty_byte_password? 20 : 16);
 
-	msg_data = (char*)malloc(password_length + 2);
+	msg_data = (uint8_t*)malloc(password_length + 2);
 
 
 	memset(&req, 0, sizeof(req));
@@ -388,7 +429,7 @@ ipmi_user_set_password(
 	memset(msg_data + 2, 0, password_length);
 
 	if (password != NULL)
-		strncpy(msg_data + 2, password, password_length);
+		strncpy((char *)(msg_data + 2), password, password_length);
 
 	rsp = intf->sendrecv(intf, &req);
 
@@ -459,6 +500,7 @@ print_user_usage(void)
 	lprintf(LOG_NOTICE, "		   set password <user id> [<password>]");
 	lprintf(LOG_NOTICE, "		   disable	<user id>");
 	lprintf(LOG_NOTICE, "		   enable	<user id>");
+	lprintf(LOG_NOTICE, "		   priv  	<user id> <privile level> [<channel number>]");
 	lprintf(LOG_NOTICE, "		   test		<user id> <16|20> [<password]>\n");
 }
 
@@ -700,8 +742,40 @@ ipmi_user_main(struct ipmi_intf * intf, int argc, char ** argv)
 			print_user_usage();
 			return -1;
 		}
+
 	}
 
+	else if (strncmp(argv[0], "priv", 4) == 0)
+	{
+   	uint8_t user_id;
+   	uint8_t priv_level;
+   	uint8_t channel = 0x0e; /* Use channel running on */
+
+		if (
+            (argc != 3) &&
+            (argc != 4)
+         )
+		
+
+      if (argc == 4)
+      {
+         channel = (uint8_t)strtol(argv[3], NULL, 0);
+         channel = (channel & 0x0f);
+		}
+      
+      user_id = (uint8_t)strtol(argv[1], NULL, 0);
+
+		priv_level = (uint8_t)strtol(argv[2], NULL, 0);
+      priv_level = (priv_level & 0x0f);
+
+		if (user_id == 0)
+		{
+			lprintf(LOG_ERR, "Invalid user ID: %d", user_id);
+			return -1;
+		}
+
+		retval = ipmi_user_set_userpriv(intf,channel,user_id,priv_level);
+   }
 
 	/*
 	 * Disable / Enable
