@@ -28,10 +28,6 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #include <stdio.h>
@@ -44,23 +40,26 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-
-#ifdef __FreeBSD__
-# include <signal.h>
-# include <paths.h>
-#endif
+#include <signal.h>
 
 #include <config.h>
 
+#ifdef HAVE_PATHS_H
+# include <paths.h>
+#else
+# define _PATH_VARRUN "/var/run/"
+#endif
+
 #ifdef IPMI_INTF_OPEN
-# ifdef HAVE_OPENIPMI_H
+# if defined(HAVE_OPENIPMI_H)
 #  include <linux/compiler.h>
 #  include <linux/ipmi.h>
-#  include <sys/poll.h>
-# else /* HAVE_OPENIPMI_H */
+# elif defined(HAVE_FREEBSD_IPMI_H)
+#  include <sys/ipmi.h>
+# else
 #  include "plugins/open/open.h"
+# endif
 #  include <sys/poll.h>
-# endif	/* HAVE_OPENIPMI_H */
 #endif /* IPMI_INTF_OPEN */
 
 #include <ipmitool/helper.h>
@@ -72,6 +71,9 @@
 #include <ipmitool/ipmi_strings.h>
 #include <ipmitool/ipmi_main.h>
 
+#define DEFAULT_PIDFILE		_PATH_VARRUN "ipmievd.pid"
+char pidfile[64];
+
 /* global variables */
 int verbose = 0;
 int csv_output = 0;
@@ -197,10 +199,10 @@ log_event(struct ipmi_event_intf * eintf, struct sel_event_record * evt)
 		return;
 	}
 
-	type = ipmi_sel_get_sensor_type_offset(evt->sensor_type, evt->event_data[0]);
-	ipmi_get_event_desc(evt, &desc);
+	type = ipmi_sel_get_sensor_type_offset(evt->sel_type.standard_type.sensor_type, evt->sel_type.standard_type.event_data[0]);
+	ipmi_get_event_desc(intf, evt, &desc);
 
-	sdr = ipmi_sdr_find_sdr_bynumtype(intf, evt->sensor_num, evt->sensor_type);
+	sdr = ipmi_sdr_find_sdr_bynumtype(intf, evt->sel_type.standard_type.sensor_num, evt->sel_type.standard_type.sensor_type);
 	if (sdr == NULL) {
 		/* could not find matching SDR record */
 		if (desc) {
@@ -209,28 +211,28 @@ log_event(struct ipmi_event_intf * eintf, struct sel_event_record * evt)
 			free(desc);
 		} else {
 			lprintf(LOG_NOTICE, "%s sensor %02x",
-				type, evt->sensor_num);
+				type, evt->sel_type.standard_type.sensor_num);
 		}
 		return;
 	}
 
 	switch (sdr->type) {
 	case SDR_RECORD_TYPE_FULL_SENSOR:
-		if (evt->event_type == 1) {
+		if (evt->sel_type.standard_type.event_type == 1) {
 			/*
 			 * Threshold Event
 			 */
 
 			/* trigger reading in event data byte 2 */
-			if (((evt->event_data[0] >> 6) & 3) == 1) {
+			if (((evt->sel_type.standard_type.event_data[0] >> 6) & 3) == 1) {
 				trigger_reading = sdr_convert_sensor_reading(
-					sdr->record.full, evt->event_data[1]);
+					sdr->record.full, evt->sel_type.standard_type.event_data[1]);
 			}
 
 			/* trigger threshold in event data byte 3 */
-			if (((evt->event_data[0] >> 4) & 3) == 1) {
+			if (((evt->sel_type.standard_type.event_data[0] >> 4) & 3) == 1) {
 				threshold_reading = sdr_convert_sensor_reading(
-					sdr->record.full, evt->event_data[2]);
+					sdr->record.full, evt->sel_type.standard_type.event_data[2]);
 			}
 
 			lprintf(LOG_NOTICE, "%s sensor %s %s (Reading %.*f %s Threshold %.*f %s)",
@@ -239,25 +241,25 @@ log_event(struct ipmi_event_intf * eintf, struct sel_event_record * evt)
 				desc ? : "",
 				(trigger_reading==(int)trigger_reading) ? 0 : 2,
 				trigger_reading,
-				((evt->event_data[0] & 0xf) % 2) ? ">" : "<",
+				((evt->sel_type.standard_type.event_data[0] & 0xf) % 2) ? ">" : "<",
 				(threshold_reading==(int)threshold_reading) ? 0 : 2,
 				threshold_reading,
 				ipmi_sdr_get_unit_string(sdr->record.full->unit.modifier,
 							 sdr->record.full->unit.type.base,
 							 sdr->record.full->unit.type.modifier));
 		}
-		else if ((evt->event_type >= 0x2 && evt->event_type <= 0xc) ||
-			 (evt->event_type == 0x6f)) {
+		else if ((evt->sel_type.standard_type.event_type >= 0x2 && evt->sel_type.standard_type.event_type <= 0xc) ||
+			 (evt->sel_type.standard_type.event_type == 0x6f)) {
 			/*
 			 * Discrete Event
 			 */
 			lprintf(LOG_NOTICE, "%s sensor %s %s",
 				type, sdr->record.full->id_string, desc ? : "");
-			if (((evt->event_data[0] >> 6) & 3) == 1) {
+			if (((evt->sel_type.standard_type.event_data[0] >> 6) & 3) == 1) {
 				/* previous state and/or severity in event data byte 2 */
 			}
 		}
-		else if (evt->event_type >= 0x70 && evt->event_type <= 0x7f) {
+		else if (evt->sel_type.standard_type.event_type >= 0x70 && evt->sel_type.standard_type.event_type <= 0x7f) {
 			/*
 			 * OEM Event
 			 */
@@ -267,13 +269,20 @@ log_event(struct ipmi_event_intf * eintf, struct sel_event_record * evt)
 		break;
 
 	case SDR_RECORD_TYPE_COMPACT_SENSOR:
-		lprintf(LOG_NOTICE, "%s sensor %s - %s",
-			type, sdr->record.compact->id_string, desc ? : "");
+		if (evt->sel_type.standard_type.event_type == 0x6f) {
+			lprintf(LOG_NOTICE, "%s sensor %s - %s %s",
+				type, sdr->record.compact->id_string,
+				desc ? : "",
+				evt->sel_type.standard_type.event_dir ? "Deasserted" : "Asserted");
+		} else {
+			lprintf(LOG_NOTICE, "%s sensor %s - %s",
+				type, sdr->record.compact->id_string, desc ? : "");
+		}
 		break;
 
 	default:
 		lprintf(LOG_NOTICE, "%s sensor - %s",
-			type, evt->sensor_num, desc ? : "");
+			type, evt->sel_type.standard_type.sensor_num, desc ? : "");
 		break;
 	}
 
@@ -364,7 +373,7 @@ openipmi_read(struct ipmi_event_intf * eintf)
 	uint8_t data[80];
 	int rv;
 
-	recv.addr = (char *) &addr;
+	recv.addr = (unsigned char *) &addr;
 	recv.addr_len = sizeof(addr);
 	recv.msg.data = data;
 	recv.msg.data_len = sizeof(data);
@@ -577,11 +586,28 @@ selwatch_wait(struct ipmi_event_intf * eintf)
 }
 /*************************************************************************/
 
+static void
+ipmievd_cleanup(int signal)
+{
+	struct stat st1;
+
+	if (lstat(pidfile, &st1) == 0) {
+		/* cleanup daemon pidfile */
+		(void)unlink(pidfile);
+	}
+
+	exit(EXIT_SUCCESS);
+}
+
 int
 ipmievd_main(struct ipmi_event_intf * eintf, int argc, char ** argv)
 {
 	int i, rc;
 	int daemon = 1;
+	struct sigaction act;
+
+	memset(pidfile, 0, 64);
+	strncpy(pidfile, DEFAULT_PIDFILE, strlen(DEFAULT_PIDFILE));
 
 	for (i = 0; i < argc; i++) {
 		if (strncasecmp(argv[i], "help", 4) == 0) {
@@ -605,11 +631,41 @@ ipmievd_main(struct ipmi_event_intf * eintf, int argc, char ** argv)
 		else if (strncasecmp(argv[i], "timeout=", 8) == 0) {
 			selwatch_timeout = strtoul(argv[i]+8, NULL, 0);
 		}
+		else if (strncasecmp(argv[i], "pidfile=", 8) == 0) {
+			memset(pidfile, 0, 64);
+			strncpy(pidfile, argv[i]+8,
+				__min(strlen((const char *)(argv[i]+8)), 63));
+		}
 	}
 
-	if (daemon)
+	if (daemon) {
+		FILE *fp;
+		struct stat st1;
+
 		ipmi_start_daemon();
 
+		if (lstat(pidfile, &st1) == 0) {
+			/* already exists, erase first */
+			if (unlink(pidfile) != 0) {
+				lprintf(LOG_WARN, "Unable to erase pidfile");
+			}
+		}
+
+		fp = ipmi_open_file_write(pidfile);
+		if (fp != NULL) {
+			fprintf(fp, "%d\n", (int)getpid());
+			fclose(fp);
+		}
+	}
+
+	/* register signal handler for cleanup */
+	act.sa_handler = ipmievd_cleanup;
+	act.sa_flags = 0;
+	sigemptyset(&act.sa_mask);
+	sigaction(SIGINT, &act, NULL);
+	sigaction(SIGQUIT, &act, NULL);
+	sigaction(SIGTERM, &act, NULL);
+
 	log_halt();
 	log_init("ipmievd", daemon, verbose);
 
@@ -619,6 +675,7 @@ ipmievd_main(struct ipmi_event_intf * eintf, int argc, char ** argv)
 	lprintf(LOG_DEBUG, "Sensors cached");
 
 	/* call event handler setup routine */
+
 	if (eintf->setup != NULL) {
 		rc = eintf->setup(eintf);
 		if (rc < 0) {
