@@ -31,10 +31,6 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #include <ipmitool/ipmi.h>
@@ -46,18 +42,22 @@
 #include <ipmitool/ipmi_fru.h>
 #include <ipmitool/ipmi_sol.h>
 #include <ipmitool/ipmi_isol.h>
+#include <ipmitool/ipmi_tsol.h>
 #include <ipmitool/ipmi_lanp.h>
 #include <ipmitool/ipmi_chassis.h>
 #include <ipmitool/ipmi_mc.h>
 #include <ipmitool/ipmi_sensor.h>
 #include <ipmitool/ipmi_channel.h>
 #include <ipmitool/ipmi_session.h>
+#include <ipmitool/ipmi_firewall.h>
 #include <ipmitool/ipmi_event.h>
 #include <ipmitool/ipmi_user.h>
 #include <ipmitool/ipmi_raw.h>
 #include <ipmitool/ipmi_pef.h>
 #include <ipmitool/ipmi_oem.h>
 #include <ipmitool/ipmi_sunoem.h>
+#include <ipmitool/ipmi_fwum.h>
+#include <ipmitool/ipmi_picmg.h>
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -66,6 +66,7 @@
 #ifdef HAVE_READLINE
 extern int ipmi_shell_main(struct ipmi_intf * intf, int argc, char ** argv);
 #endif
+extern int ipmi_echo_main(struct ipmi_intf * intf, int argc, char ** argv);
 extern int ipmi_set_main(struct ipmi_intf * intf, int argc, char ** argv);
 extern int ipmi_exec_main(struct ipmi_intf * intf, int argc, char ** argv);
 
@@ -78,6 +79,7 @@ struct ipmi_cmd ipmitool_cmd_list[] = {
 	{ ipmi_rawi2c_main,  "i2c",     "Send an I2C Master Write-Read command and print response" },
 	{ ipmi_lanp_main,    "lan",     "Configure LAN Channels" },
 	{ ipmi_chassis_main, "chassis", "Get chassis status and set power state" },
+	{ ipmi_power_main,   "power",   "Shortcut to chassis power commands" },
 	{ ipmi_event_main,   "event",   "Send pre-defined events to MC" },
 	{ ipmi_mc_main,      "mc",      "Management Controller status and global enables" },
 	{ ipmi_mc_main,      "bmc",     NULL },	/* for backwards compatibility */
@@ -86,17 +88,22 @@ struct ipmi_cmd ipmitool_cmd_list[] = {
 	{ ipmi_fru_main,     "fru",     "Print built-in FRU and scan SDR for FRU locators" },
 	{ ipmi_sel_main,     "sel",     "Print System Event Log (SEL)" },
 	{ ipmi_pef_main,     "pef",     "Configure Platform Event Filtering (PEF)" },
-	{ ipmi_sol_main,     "sol",     "Configure IPMIv2.0 Serial-over-LAN" },
+	{ ipmi_sol_main,     "sol",     "Configure and connect IPMIv2.0 Serial-over-LAN" },
+	{ ipmi_tsol_main,    "tsol",    "Configure and connect with Tyan IPMIv1.5 Serial-over-LAN" },
 	{ ipmi_isol_main,    "isol",    "Configure IPMIv1.5 Serial-over-LAN" },
 	{ ipmi_user_main,    "user",    "Configure Management Controller users" },
 	{ ipmi_channel_main, "channel", "Configure Management Controller channels" },
 	{ ipmi_session_main, "session", "Print session information" },
+	{ ipmi_firewall_main,"firewall","Configure firmware firewall (IPMIv2.0)"},
 	{ ipmi_sunoem_main,  "sunoem",  "OEM Commands for Sun servers" },
+	{ ipmi_picmg_main,   "picmg",   "Run a PICMG/ATCA extended cmd"},
+	{ ipmi_fwum_main,    "fwum",	"Update IPMC using Kontron OEM Firmware Update Manager" },
 #ifdef HAVE_READLINE
 	{ ipmi_shell_main,   "shell",   "Launch interactive IPMI shell" },
 #endif
 	{ ipmi_exec_main,    "exec",    "Run list of commands from file" },
 	{ ipmi_set_main,     "set",     "Set runtime variable for shell and exec" },
+	{ ipmi_echo_main,    "echo",    NULL }, /* for echoing lines to stdout in scripts */
 	{ NULL },
 };
 
