@@ -28,10 +28,6 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #include <stdio.h>
@@ -180,7 +176,7 @@ ipmi_intf_session_set_username(struct ipmi_intf * intf, char * username)
 	if (intf->session == NULL)
 		return;
 
-	memset(intf->session->username, 0, 16);
+	memset(intf->session->username, 0, 17);
 
 	if (username == NULL)
 		return;
@@ -224,6 +220,30 @@ ipmi_intf_session_set_cipher_suite_id(struct ipmi_intf * intf, uint8_t cipher_su
 	intf->session->cipher_suite_id = cipher_suite_id;
 }
 
+void
+ipmi_intf_session_set_sol_escape_char(struct ipmi_intf * intf, char sol_escape_char)
+{
+	if (intf->session == NULL)
+		return;
+
+	intf->session->sol_escape_char = sol_escape_char;
+}
+
+void
+ipmi_intf_session_set_kgkey(struct ipmi_intf * intf, char * kgkey)
+{
+	if (intf->session == NULL)
+		return;
+
+	memset(intf->session->v2_data.kg, 0, IPMI_KG_BUFFER_SIZE);
+
+	if (kgkey == NULL)
+		return;
+
+	memcpy(intf->session->v2_data.kg, kgkey, 
+	       __min(strlen(kgkey), IPMI_KG_BUFFER_SIZE));
+}
+
 void
 ipmi_intf_session_set_port(struct ipmi_intf * intf, int port)
 {
