@@ -28,10 +28,6 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #include <stdlib.h>
@@ -75,6 +71,8 @@ static struct ipmi_rq_entry * ipmi_req_entries_tail;
 
 extern int verbose;
 
+static uint8_t bridgePossible = 0; 
+
 static sigjmp_buf jmpbuf;
 
 static int ipmi_lan_send_packet(struct ipmi_intf * intf, uint8_t * data, int data_len);
@@ -134,7 +132,7 @@ ipmi_req_lookup_entry(uint8_t seq, uint8_t cmd)
 {
 	struct ipmi_rq_entry * e = ipmi_req_entries;
 	while (e && (e->rq_seq != seq || e->req.msg.cmd != cmd)) {
-		if (e == e->next)
+		if (e->next == NULL || e == e->next)
 			return NULL;
 		e = e->next;
 	}
@@ -183,10 +181,16 @@ ipmi_req_clear_entries(void)
 	while (e) {
 		lprintf(LOG_DEBUG+3, "cleared list entry seq=0x%02x cmd=0x%02x",
 			e->rq_seq, e->req.msg.cmd);
-		p = e->next;
-		free(e);
-		e = p;
+		if (e->next != NULL) {
+			p = e->next;
+			free(e);
+			e = p;
+		} else {
+			free(e);
+			break;
+		}
 	}
+	ipmi_req_entries = NULL;
 }
 
 static int
@@ -392,6 +396,10 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 	struct ipmi_rs * rsp;
 	struct ipmi_rq_entry * entry;
 	int x=0, rv;
+	uint8_t ourAddress = intf->my_addr;
+
+	if (ourAddress == 0)
+		ourAddress = IPMI_BMC_SLAVE_ADDR;
 
 	rsp = ipmi_lan_recv_packet(intf);
 
@@ -470,16 +478,19 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 					      rsp->payload.ipmi_response.cmd);
 		if (entry) {
 			lprintf(LOG_DEBUG+2, "IPMI Request Match found");
-			if (intf->target_addr != IPMI_BMC_SLAVE_ADDR) {
-				if (rsp->data_len)
-					lprintf(LOG_DEBUG+1, "Bridged cmd %02x resp: %s",
-						rsp->payload.ipmi_response.cmd,
-						buf2str(&rsp->data[x],rsp->data_len));
+			if ((intf->target_addr != ourAddress) && bridgePossible) {
+				if ((rsp->data_len) &&
+				    (rsp->payload.ipmi_response.cmd != 0x34)) {
+					printbuf(&rsp->data[x], rsp->data_len-x,
+						 "bridge command response");
+				}
 				/* bridged command: lose extra header */
 				if (rsp->payload.ipmi_response.cmd == 0x34) {
-					entry->req.msg.cmd = entry->req.msg.target_cmd;
-					rsp = ipmi_lan_recv_packet(intf);
-					continue;
+					if (rsp->data_len == 38) {
+						entry->req.msg.cmd = entry->req.msg.target_cmd;
+						rsp = ipmi_lan_recv_packet(intf);
+						continue;
+					}
 				} else {
 					//x += sizeof(rsp->payload.ipmi_response);
 					if (rsp->data[x-1] != 0)
@@ -494,7 +505,7 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 			lprintf(LOG_INFO, "IPMI Request Match NOT FOUND");
 			rsp = ipmi_lan_recv_packet(intf);
 			continue;
-		}			
+		}
 
 		break;
 	}
@@ -553,6 +564,11 @@ ipmi_lan_build_cmd(struct ipmi_intf * intf, struct ipmi_rq * req)
 	struct ipmi_rq_entry * entry;
 	struct ipmi_session * s = intf->session;
 	static int curr_seq = 0;
+	uint8_t ourAddress = intf->my_addr;
+	uint8_t bridgedRequest = 0;
+
+	if (ourAddress == 0)
+		ourAddress = IPMI_BMC_SLAVE_ADDR;
 
 	if (curr_seq >= 64)
 		curr_seq = 0;
@@ -560,7 +576,7 @@ ipmi_lan_build_cmd(struct ipmi_intf * intf, struct ipmi_rq * req)
 	entry = ipmi_req_add_entry(intf, req);
 	if (entry == NULL)
 		return NULL;
-	
+
 	len = req->msg.data_len + 29;
 	if (s->active && s->authtype)
 		len += 16;
@@ -593,12 +609,12 @@ ipmi_lan_build_cmd(struct ipmi_intf * intf, struct ipmi_rq * req)
 	}
 
 	/* message length */
-	if (intf->target_addr == IPMI_BMC_SLAVE_ADDR) {
+	if ((intf->target_addr == ourAddress) || !bridgePossible) {
 		msg[len++] = req->msg.data_len + 7;
 		cs = mp = len;
-	}
-	else {
+	} else {
 		/* bridged request: encapsulate w/in Send Message */
+		bridgedRequest = 1;
 		msg[len++] = req->msg.data_len + 15;
 		cs = mp = len;
 		msg[len++] = IPMI_BMC_SLAVE_ADDR;
@@ -611,17 +627,21 @@ ipmi_lan_build_cmd(struct ipmi_intf * intf, struct ipmi_rq * req)
 		msg[len++] = 0x34;			/* Send Message rqst */
 		entry->req.msg.target_cmd = entry->req.msg.cmd;	/* Save target command */
 		entry->req.msg.cmd = 0x34;		/* (fixup request entry) */
-		msg[len++] = 0x40;			/* Track request, Channel=IPMB */
+		msg[len++] = (0x40|intf->target_channel); /* Track request*/
 		cs = len;
 	}
 
 	/* ipmi message header */
 	msg[len++] = intf->target_addr;
-	msg[len++] = req->msg.netfn << 2;
+	msg[len++] = req->msg.netfn << 2 | (req->msg.lun & 3);
 	tmp = len - cs;
 	msg[len++] = ipmi_csum(msg+cs, tmp);
 	cs = len;
-	msg[len++] = IPMI_REMOTE_SWID;
+
+	if (!bridgedRequest)
+		msg[len++] = IPMI_REMOTE_SWID;
+	else  /* Bridged message */
+		msg[len++] = intf->my_addr;
 
 	entry->rq_seq = curr_seq++;
 	msg[len++] = entry->rq_seq << 2;
@@ -652,7 +672,7 @@ ipmi_lan_build_cmd(struct ipmi_intf * intf, struct ipmi_rq * req)
 	msg[len++] = ipmi_csum(msg+cs, tmp);
 
 	/* bridged request: 2nd checksum */
-	if (intf->target_addr != IPMI_BMC_SLAVE_ADDR) {
+	if (bridgedRequest) {
 		tmp = len - cs2;
 		msg[len++] = ipmi_csum(msg+cs2, tmp);
 	}
@@ -694,9 +714,17 @@ ipmi_lan_send_cmd(struct ipmi_intf * intf, struct ipmi_rq * req)
 	struct ipmi_rs * rsp = NULL;
 	int try = 0;
 
+	lprintf(LOG_DEBUG, "ipmi_lan_send_cmd:opened=[%d], open=[%d]",
+		intf->opened, intf->open);
+
 	if (intf->opened == 0 && intf->open != NULL) {
-		if (intf->open(intf) < 0)
+		if (intf->open(intf) < 0) {
+			lprintf(LOG_ERR,
+				"ipmi_lan_send_cmd failed to open intf");
 			return NULL;
+		}
+		lprintf(LOG_DEBUG, "\topened=[%d], open=[%d]",
+			intf->opened, intf->open);
 	}
 
 	entry = ipmi_lan_build_cmd(intf, req);
@@ -1143,6 +1171,8 @@ ipmi_activate_session_cmd(struct ipmi_intf * intf)
 		return -1;
 	}
 
+   bridgePossible = 1;
+   
 	lprintf(LOG_DEBUG, "\nSession Activated");
 	lprintf(LOG_DEBUG, "  Auth Type       : %s",
 		val2str(rsp->data[0], ipmi_authtype_session_vals));
@@ -1164,6 +1194,7 @@ ipmi_set_session_privlvl_cmd(struct ipmi_intf * intf)
 	struct ipmi_rs * rsp;
 	struct ipmi_rq req;
 	uint8_t privlvl = intf->session->privlvl;
+   uint8_t backupBridgePossible = bridgePossible;
 
 	if (privlvl <= IPMI_SESSION_PRIV_USER)
 		return 0;	/* no need to set higher */
@@ -1174,7 +1205,12 @@ ipmi_set_session_privlvl_cmd(struct ipmi_intf * intf)
 	req.msg.data		= &privlvl;
 	req.msg.data_len	= 1;
 
+   bridgePossible = 0;
+
 	rsp = intf->sendrecv(intf, &req);
+   
+   bridgePossible = backupBridgePossible;
+
 	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Set Session Privilege Level to %s failed",
 			val2str(privlvl, ipmi_privlvl_vals));
@@ -1208,6 +1244,7 @@ ipmi_close_session_cmd(struct ipmi_intf * intf)
 		return -1;
 
 	intf->target_addr = IPMI_BMC_SLAVE_ADDR;
+   bridgePossible = 0;  /* Not a bridge message */
 
 	memcpy(&msg_data, &session_id, 4);
 
@@ -1341,7 +1378,7 @@ int ipmi_lan_open(struct ipmi_intf * intf)
 	if (s->retry == 0)
 		s->retry = IPMI_LAN_RETRY;
 
-	if (s->hostname == NULL || strlen(s->hostname) == 0) {
+	if (s->hostname == NULL || strlen((const char *)s->hostname) == 0) {
 		lprintf(LOG_ERR, "No hostname specified!");
 		return -1;
 	}
@@ -1353,13 +1390,9 @@ int ipmi_lan_open(struct ipmi_intf * intf)
 	s->addr.sin_family = AF_INET;
 	s->addr.sin_port = htons(s->port);
 
-#ifdef __CYGWIN__
-	rc = inet_aton(s->hostname, &s->addr.sin_addr);
-#else
-	rc = inet_pton(AF_INET, s->hostname, &s->addr.sin_addr);
-#endif
+	rc = inet_pton(AF_INET, (const char *)s->hostname, &s->addr.sin_addr);
 	if (rc <= 0) {
-		struct hostent *host = gethostbyname(s->hostname);
+		struct hostent *host = gethostbyname((const char *)s->hostname);
 		if (host == NULL) {
 			lprintf(LOG_ERR, "Address lookup for %s failed",
 				s->hostname);
