@@ -28,10 +28,6 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #include <stdlib.h>
@@ -92,10 +88,14 @@ static struct ipmi_rs * ipmi_lanplus_send_ipmi_cmd(struct ipmi_intf * intf, stru
 static struct ipmi_rs * ipmi_lanplus_send_payload(struct ipmi_intf * intf,
 												  struct ipmi_v2_payload * payload);
 static void getIpmiPayloadWireRep(
+						 		  struct ipmi_intf       * intf,
+						 		  struct ipmi_v2_payload * payload,  /* in  */
 								  uint8_t  * out,
 								  struct ipmi_rq * req,
-								  uint8_t    rq_seq);
+								  uint8_t    rq_seq,
+					 			  uint8_t curr_seq);
 static void getSolPayloadWireRep(
+						 		  struct ipmi_intf       * intf,
 								 uint8_t          * msg,
 								 struct ipmi_v2_payload * payload);
 static void read_open_session_response(struct ipmi_rs * rsp, int offset);
@@ -117,6 +117,7 @@ static void ack_sol_packet(
 						   struct ipmi_intf * intf,
 						   struct ipmi_rs * rsp);
 
+static uint8_t bridgePossible = 0; 
 
 struct ipmi_intf ipmi_lanplus_intf = {
 	name:		"lanplus",
@@ -287,7 +288,7 @@ static const struct valstr plus_payload_types_vals[] = {
 
 
 static struct ipmi_rq_entry *
-ipmi_req_add_entry(struct ipmi_intf * intf, struct ipmi_rq * req)
+ipmi_req_add_entry(struct ipmi_intf * intf, struct ipmi_rq * req, uint8_t req_seq)
 {
 	struct ipmi_rq_entry * e;
 
@@ -301,6 +302,7 @@ ipmi_req_add_entry(struct ipmi_intf * intf, struct ipmi_rq * req)
 	memcpy(&e->req, req, sizeof(struct ipmi_rq));
 
 	e->intf = intf;
+   e->rq_seq = req_seq;
 
 	if (ipmi_req_entries == NULL)
 		ipmi_req_entries = e;
@@ -583,6 +585,13 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 	int offset, rv;
 	uint16_t payload_size;
 
+   uint8_t ourAddress = intf->my_addr;
+   
+   if(ourAddress == 0)
+   {
+      ourAddress = IPMI_BMC_SLAVE_ADDR;
+   }
+
 	rsp = ipmi_lan_recv_packet(intf);
 
 	/*
@@ -701,6 +710,37 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 										  rsp->payload.ipmi_response.cmd);
 			if (entry != NULL) {
 				lprintf(LOG_DEBUG+2, "IPMI Request Match found");
+            if (
+                  (intf->target_addr != ourAddress) &&
+                  (bridgePossible)
+               )
+            {
+				   if(
+                  (rsp->data_len) &&
+                  (rsp->payload.ipmi_response.cmd != 0x34)
+                 )
+				   {
+      			   printbuf(
+                           &rsp->data[offset], 
+                           (rsp->data_len-offset),
+                           "bridge command response");
+				   }
+				   /* bridged command: lose extra header */
+				   if (rsp->payload.ipmi_response.cmd == 0x34) {
+                  if( rsp->data_len == 38 )
+                  {
+					   entry->req.msg.cmd = entry->req.msg.target_cmd;
+					   rsp = ipmi_lan_recv_packet(intf);
+					   continue;
+                  }
+				   } else {
+					   //x += sizeof(rsp->payload.ipmi_response);
+					   if (rsp->data[offset-1] != 0)
+						   lprintf(LOG_DEBUG, "WARNING: Bridged "
+							   "cmd ccode = 0x%02x",
+						          rsp->data[offset-1]);
+				   }
+			   }
 				ipmi_req_remove_entry(rsp->payload.ipmi_response.rq_seq,
 									  rsp->payload.ipmi_response.cmd);
 			} else {
@@ -1283,48 +1323,88 @@ void read_sol_packet(struct ipmi_rs * rsp, int * offset)
  * param rq_seq [in] is the IPMI command sequence number.
  */
 void getIpmiPayloadWireRep(
+						   struct ipmi_intf       * intf,  /* in out */
+						   struct ipmi_v2_payload * payload,  /* in  */
 						   uint8_t  * msg,
 						   struct ipmi_rq * req,
-						   uint8_t    rq_seq)
+						   uint8_t    rq_seq,
+						   uint8_t curr_seq)
 {
-	int cs, mp, tmp, i;
+	int cs, tmp, len;
+	int cs2 = 0;
+	uint8_t ourAddress = intf->my_addr;
+	uint8_t bridgedRequest = 0;
 
-	i = 0;
+	if (ourAddress == 0)
+		ourAddress = IPMI_BMC_SLAVE_ADDR;
+
+	len = 0;
 
 	/* IPMI Message Header -- Figure 13-4 of the IPMI v2.0 spec */
-	cs = mp = i,
+	if ((intf->target_addr == ourAddress) || (!bridgePossible))
+		cs = len;
+	else {
+		/* bridged request: encapsulate w/in Send Message */
+		bridgedRequest = 1;
+		cs = len;
+ 		msg[len++] = IPMI_BMC_SLAVE_ADDR;
+		msg[len++] = IPMI_NETFN_APP << 2;
+		tmp = len - cs;
+		msg[len++] = ipmi_csum(msg+cs, tmp);
+		cs2 = len;
+		msg[len++] = IPMI_REMOTE_SWID;
+		msg[len++] = curr_seq << 2;
+		msg[len++] = 0x34;			/* Send Message rqst */
+#if 0  /* From lan.c example */
+		entry->req.msg.target_cmd = entry->req.msg.cmd;	/* Save target command */
+		entry->req.msg.cmd = 0x34;		/* (fixup request entry) */
+#endif
+		msg[len++] = (0x40|intf->target_channel); /* Track request*/
+
+		payload->payload_length += 7;
+
+		cs = len;
+	}
 
 	/* rsAddr */
-	msg[i++] = IPMI_BMC_SLAVE_ADDR; 
+	msg[len++] = intf->target_addr; /* IPMI_BMC_SLAVE_ADDR; */
 
 	/* net Fn */
-	msg[i++] = req->msg.netfn << 2;
-	tmp = i - cs;
+	msg[len++] = req->msg.netfn << 2 | (req->msg.lun & 3);
+	tmp = len - cs;
 
 	/* checkSum */
-	msg[i++] = ipmi_csum(msg+cs, tmp);
-	cs = i;
+	msg[len++] = ipmi_csum(msg+cs, tmp);
+	cs = len;
 
 	/* rqAddr */
-	msg[i++] = IPMI_REMOTE_SWID;
+	if (!bridgedRequest)
+		msg[len++] = IPMI_REMOTE_SWID;
+	else  /* Bridged message */
+		msg[len++] = intf->my_addr;
 
 	/* rqSeq / rqLUN */
-	msg[i++] = rq_seq << 2;
+	msg[len++] = rq_seq << 2;
 
 	/* cmd */
-	msg[i++] = req->msg.cmd;
+	msg[len++] = req->msg.cmd;
 
 	/* message data */
 	if (req->msg.data_len) {
-		memcpy(msg + i,
-			   req->msg.data,
-			   req->msg.data_len);
-		i += req->msg.data_len;
+		memcpy(msg + len, req->msg.data, req->msg.data_len);
+		len += req->msg.data_len;
 	}
 
 	/* second checksum */
-	tmp = i - cs;
-	msg[i++] = ipmi_csum(msg+cs, tmp);
+	tmp = len - cs;
+	msg[len++] = ipmi_csum(msg+cs, tmp);
+
+   	/* bridged request: 2nd checksum */
+	if (bridgedRequest) {
+		tmp = len - cs2;
+		msg[len++] = ipmi_csum(msg+cs2, tmp);
+		payload->payload_length += 1;
+	}
 }
 
 
@@ -1336,6 +1416,7 @@ void getIpmiPayloadWireRep(
  * param payload [in] holds the v2 payload with our SOL data
  */
 void getSolPayloadWireRep(
+						  struct ipmi_intf       * intf,  /* in out */
 						  uint8_t          * msg,     /* output */
 						  struct ipmi_v2_payload * payload) /* input */
 {
@@ -1408,7 +1489,8 @@ ipmi_lanplus_build_v2x_msg(
 						   struct ipmi_intf       * intf,     /* in  */
 						   struct ipmi_v2_payload * payload,  /* in  */
 						   int                    * msg_len,  /* out */
-						   uint8_t         ** msg_data) /* out */
+						   uint8_t         ** msg_data, /* out */
+						   uint8_t curr_seq)
 {
 	uint32_t session_trailer_length = 0;
 	struct ipmi_session * session = intf->session;
@@ -1505,13 +1587,17 @@ ipmi_lanplus_build_v2x_msg(
 	switch (payload->payload_type)
 	{
 	case IPMI_PAYLOAD_TYPE_IPMI:
-		getIpmiPayloadWireRep(msg + IPMI_LANPLUS_OFFSET_PAYLOAD,
+		getIpmiPayloadWireRep(intf,
+                       payload,  /* in  */
+							  msg + IPMI_LANPLUS_OFFSET_PAYLOAD,
 							  payload->payload.ipmi_request.request,
-							  payload->payload.ipmi_request.rq_seq);
+							  payload->payload.ipmi_request.rq_seq,
+                       curr_seq);
 		break;
 
 	case IPMI_PAYLOAD_TYPE_SOL: 
-		getSolPayloadWireRep(msg + IPMI_LANPLUS_OFFSET_PAYLOAD,
+		getSolPayloadWireRep(intf,
+							 msg + IPMI_LANPLUS_OFFSET_PAYLOAD,
 							 payload);
 		len += payload->payload_length;
 
@@ -1698,10 +1784,13 @@ ipmi_lanplus_build_v2x_ipmi_cmd(
 	 * representation far below.
 	 */
  	static uint8_t curr_seq = 0;
+   
+   curr_seq += 1;
+   
 	if (curr_seq >= 64)
 		curr_seq = 0;
 
-	entry = ipmi_req_add_entry(intf, req);
+	entry = ipmi_req_add_entry(intf, req, curr_seq);
 	if (entry == NULL)
 		return NULL;
 
@@ -1714,7 +1803,8 @@ ipmi_lanplus_build_v2x_ipmi_cmd(
 	ipmi_lanplus_build_v2x_msg(intf,                // in 
 							   &v2_payload,         // in 
 							   &(entry->msg_len),   // out
-							   &(entry->msg_data)); // out
+							   &(entry->msg_data),  // out
+                        curr_seq); // in
 
 	return entry;
 }
@@ -1766,7 +1856,7 @@ ipmi_lanplus_build_v15_ipmi_cmd(
 	struct ipmi_session  * session = intf->session;
 	struct ipmi_rq_entry * entry;
 
-	entry = ipmi_req_add_entry(intf, req);
+	entry = ipmi_req_add_entry(intf, req, 0);
 	if (entry == NULL)
 		return NULL;
 
@@ -1965,7 +2055,8 @@ ipmi_lanplus_send_payload(
 		ipmi_lanplus_build_v2x_msg(intf,        /* in  */
 								   payload,     /* in  */
 								   &msg_length, /* out */
-								   &msg_data);  /* out */
+								   &msg_data,   /* out */
+                           0);  /* irrelevant for this msg*/
 
 	}
 
@@ -1978,7 +2069,8 @@ ipmi_lanplus_send_payload(
 		ipmi_lanplus_build_v2x_msg(intf,        /* in  */
 								   payload,     /* in  */
 								   &msg_length, /* out */
-								   &msg_data);  /* out */
+								   &msg_data,   /* out */
+                           0);  /* irrelevant for this msg*/
 
 	}
 
@@ -1991,7 +2083,8 @@ ipmi_lanplus_send_payload(
 		ipmi_lanplus_build_v2x_msg(intf,        /* in  */
 								   payload,     /* in  */
 								   &msg_length, /* out */
-								   &msg_data);  /* out */
+								   &msg_data,   /* out */
+                           0);  /* irrelevant for this msg*/
 
 	}
 
@@ -2003,7 +2096,8 @@ ipmi_lanplus_send_payload(
 		ipmi_lanplus_build_v2x_msg(intf,        /* in  */
 								   payload,     /* in  */
 								   &msg_length, /* out */
-								   &msg_data);  /* out */
+								   &msg_data,   /* out */
+                           0);  /* irrelevant for this msg*/
 	}
 
 	else
@@ -2094,9 +2188,10 @@ ipmi_lanplus_send_payload(
 			if (rsp)
 				break;
 
-			usleep(5000);
 		}
 
+		usleep(5000);
+
 		try++;
 	}
 
@@ -2192,6 +2287,8 @@ ipmi_lanplus_send_sol(
 	 * Payload length is just the length of the character
 	 * data here.
 	 */
+	v2_payload->payload_length = v2_payload->payload.sol_packet.character_count;
+
 	v2_payload->payload.sol_packet.acked_packet_number = 0; /* NA */
 
 	set_sol_packet_sequence_number(intf, v2_payload);
@@ -2223,6 +2320,8 @@ ipmi_lanplus_send_sol(
 
 		v2_payload->payload.sol_packet.character_count = chars_to_resend;
 
+		v2_payload->payload_length = v2_payload->payload.sol_packet.character_count;
+
 		rs = ipmi_lanplus_send_payload(intf, v2_payload);
 
 		chars_to_resend = is_sol_partial_ack(v2_payload, rs);
@@ -2416,6 +2515,11 @@ ipmi_get_auth_capabilities_cmd(
 	struct ipmi_rs * rsp;
 	struct ipmi_rq req;
 	uint8_t msg_data[2];
+   uint8_t backupBridgePossible;
+
+   backupBridgePossible = bridgePossible;
+
+   bridgePossible = 0;
 
 	msg_data[0] = IPMI_LAN_CHANNEL_E | 0x80; // Ask for IPMI v2 data as well
 	msg_data[1] = intf->session->privlvl;
@@ -2453,6 +2557,7 @@ ipmi_get_auth_capabilities_cmd(
 		   rsp->data,
 		   sizeof(struct get_channel_auth_cap_rsp));
 
+   bridgePossible = backupBridgePossible;
 
 	return 0;
 }
@@ -2466,10 +2571,14 @@ impi_close_session_cmd(struct ipmi_intf * intf)
 	struct ipmi_rq req;
 	uint8_t msg_data[4];
 	uint32_t bmc_session_lsbf;
+   uint8_t backupBridgePossible;
 
 	if (intf->session->v2_data.session_state != LANPLUS_STATE_ACTIVE)
 		return -1;
 
+   backupBridgePossible = bridgePossible;
+
+   bridgePossible = 0;
 
 	bmc_session_lsbf = intf->session->v2_data.bmc_id;
 	#if WORDS_BIGENDIAN
@@ -2508,6 +2617,7 @@ impi_close_session_cmd(struct ipmi_intf * intf)
 	lprintf(LOG_DEBUG, "Closed Session %08lx\n",
 		(long)intf->session->v2_data.bmc_id);
 
+   bridgePossible = backupBridgePossible;
 	return 0;
 }
 
@@ -2766,7 +2876,7 @@ ipmi_lanplus_rakp1(struct ipmi_intf * intf)
 
 
 	/* Username specification */
-	msg[27] = strlen(session->username);
+	msg[27] = strlen((const char *)session->username);
 	if (msg[27] > IPMI_MAX_USER_NAME_LENGTH)
 	{
 		lprintf(LOG_ERR, "ERROR: user name too long.  "
@@ -3023,6 +3133,51 @@ ipmi_lanplus_close(struct ipmi_intf * intf)
 
 
 
+static int
+ipmi_set_session_privlvl_cmd(struct ipmi_intf * intf)
+{
+	struct ipmi_rs * rsp;
+	struct ipmi_rq req;
+   uint8_t backupBridgePossible;
+	uint8_t privlvl = intf->session->privlvl;
+
+	if (privlvl <= IPMI_SESSION_PRIV_USER)
+		return 0;	/* no need to set higher */
+
+   backupBridgePossible = bridgePossible;
+
+   bridgePossible = 0;
+
+	memset(&req, 0, sizeof(req));
+	req.msg.netfn		= IPMI_NETFN_APP;
+	req.msg.cmd		= 0x3b;
+	req.msg.data		= &privlvl;
+	req.msg.data_len	= 1;
+
+	rsp = intf->sendrecv(intf, &req);
+	if (rsp == NULL) {
+		lprintf(LOG_ERR, "Set Session Privilege Level to %s failed",
+			val2str(privlvl, ipmi_privlvl_vals));
+		return -1;
+	}
+	if (verbose > 2)
+		printbuf(rsp->data, rsp->data_len, "set_session_privlvl");
+
+	if (rsp->ccode > 0) {
+		lprintf(LOG_ERR, "Set Session Privilege Level to %s failed: %s",
+			val2str(privlvl, ipmi_privlvl_vals),
+			val2str(rsp->ccode, completion_code_vals));
+		return -1;
+	}
+
+	lprintf(LOG_DEBUG, "Set Session Privilege Level to %s\n",
+		val2str(rsp->data[0], ipmi_privlvl_vals));
+
+   bridgePossible = backupBridgePossible;
+
+	return 0;
+}
+
 /**
  * ipmi_lanplus_open
  */
@@ -3049,7 +3204,7 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 	if (!session->retry)
 		session->retry = IPMI_LAN_RETRY;
 
-	if (session->hostname == NULL || strlen(session->hostname) == 0) {
+	if (session->hostname == NULL || strlen((const char *)session->hostname) == 0) {
 		lprintf(LOG_ERR, "No hostname specified!");
 		return -1;
 	}
@@ -3067,7 +3222,9 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 	//session->sol_data.last_received_sequence_number = 0;
 	//session->sol_data.last_received_byte_count      = 0;
 	memset(session->v2_data.sik, 0, IPMI_SIK_BUFFER_SIZE);
-	memset(session->v2_data.kg,  0, IPMI_KG_BUFFER_SIZE);
+
+	/* Kg is set in ipmi_intf */
+	//memset(session->v2_data.kg,  0, IPMI_KG_BUFFER_SIZE);
 
 
 	/* open port to BMC */
@@ -3075,13 +3232,9 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 	addr.sin_family = AF_INET;
 	addr.sin_port = htons(session->port);
 
-#ifdef __CYGWIN__
-	rc = inet_aton(session->hostname, &addr.sin_addr);
-#else
-	rc = inet_pton(AF_INET, session->hostname, &addr.sin_addr);
-#endif
+	rc = inet_pton(AF_INET, (const char *)session->hostname, &addr.sin_addr);
 	if (rc <= 0) {
-		struct hostent *host = gethostbyname(session->hostname);
+		struct hostent *host = gethostbyname((const char *)session->hostname);
 		if (host == NULL) {
 			lprintf(LOG_ERR, "Address lookup for %s failed",
 				session->hostname);
@@ -3174,6 +3327,12 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 
 	lprintf(LOG_DEBUG, "IPMIv2 / RMCP+ SESSION OPENED SUCCESSFULLY\n");
 
+   bridgePossible = 1;
+	
+	rc = ipmi_set_session_privlvl_cmd(intf);
+	if (rc < 0)
+		goto fail;
+
 	return intf->fd;
 
  fail:
@@ -3241,22 +3400,22 @@ void test_crypt2(void)
 	uint8_t iv[]  =
         {0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A,
          0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, 0x11, 0x12, 0x13, 0x14};
-	uint8_t * data = "12345678";
+	uint8_t data[8] = "12345678";
 
-	char encrypt_buffer[1000];
-	char decrypt_buffer[1000];
+	uint8_t encrypt_buffer[1000];
+	uint8_t decrypt_buffer[1000];
 	uint32_t bytes_encrypted;
 	uint32_t bytes_decrypted;
 
-	printbuf(data, strlen(data), "input data");
+	printbuf((const uint8_t *)data, strlen((const char *)data), "input data");
 
 	lanplus_encrypt_aes_cbc_128(iv,
 								key,
 								data,
-								strlen(data),
+								strlen((const char *)data),
 								encrypt_buffer,
 								&bytes_encrypted);
-	printbuf(encrypt_buffer, bytes_encrypted, "encrypt_buffer");
+	printbuf((const uint8_t *)encrypt_buffer, bytes_encrypted, "encrypt_buffer");
 
 	lanplus_decrypt_aes_cbc_128(iv,
 								key,
@@ -3264,7 +3423,7 @@ void test_crypt2(void)
 								bytes_encrypted,
 								decrypt_buffer,
 								&bytes_decrypted);
-	printbuf(decrypt_buffer, bytes_decrypted, "decrypt_buffer");
+	printbuf((const uint8_t *)decrypt_buffer, bytes_decrypted, "decrypt_buffer");
 
 	lprintf(LOG_INFO, "\nDone testing the encrypt/decyrpt methods!\n");
 	exit(0);
