@@ -28,10 +28,6 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #include <stdio.h>
@@ -51,30 +47,44 @@
 
 #include <config.h>
 
-#ifdef HAVE_OPENIPMI_H
+#if defined(HAVE_OPENIPMI_H)
 # include <linux/compiler.h>
 # include <linux/ipmi.h>
+#elif defined(HAVE_FREEBSD_IPMI_H)
+/* FreeBSD OpenIPMI-compatible header */
+# include <sys/ipmi.h>
 #else
 # include "open.h"
 #endif
 
 extern int verbose;
 
-#define IPMI_OPENIPMI_DEV	"/dev/ipmi0"
+#define IPMI_OPENIPMI_DEV		"/dev/ipmi0"
 #define IPMI_OPENIPMI_DEVFS	"/dev/ipmi/0"
+#define IPMI_OPENIPMI_DEVFS2	"/dev/ipmidev/0"
 
 static int
 ipmi_openipmi_open(struct ipmi_intf * intf)
 {
 	int i = 0;
+#ifdef INCLUDE_PICMG_GET_DEVICE_LOCATOR
+	struct ipmi_rq req;
+	struct ipmi_rs *rsp;
+	char msg_data;
+#endif
 
 	intf->fd = open(IPMI_OPENIPMI_DEV, O_RDWR);
 
 	if (intf->fd < 0) {
 		intf->fd = open(IPMI_OPENIPMI_DEVFS, O_RDWR);
-		lperror(LOG_ERR, "Could not open device at %s or %s",
-			IPMI_OPENIPMI_DEV, IPMI_OPENIPMI_DEVFS);
-		return -1;
+		if (intf->fd < 0) {
+			intf->fd = open(IPMI_OPENIPMI_DEVFS2, O_RDWR);
+		}
+		if (intf->fd < 0) {
+			lperror(LOG_ERR, "Could not open device at %s or %s or %s",
+			IPMI_OPENIPMI_DEV, IPMI_OPENIPMI_DEVFS , IPMI_OPENIPMI_DEVFS2);
+			return -1;
+		}
 	}
 
 	if (ioctl(intf->fd, IPMICTL_SET_GETS_EVENTS_CMD, &i) < 0) {
@@ -91,9 +101,33 @@ ipmi_openipmi_open(struct ipmi_intf * intf)
 		lprintf(LOG_DEBUG, "Set IPMB address to 0x%x",
 			intf->my_addr);
 	}
-
 	intf->opened = 1;
 
+#ifdef INCLUDE_PICMG_GET_DEVICE_LOCATOR
+	/* PICMG hack to set right IPMB address, 
+	 * we might want to do GetPICMGProperties first.
+	 * In any case, on a server board or a non-picmg IpmC blade , this code 
+	 * will not have any adverse side effect
+	 */
+	if (intf->my_addr == IPMI_BMC_SLAVE_ADDR) {
+		lprintf(LOG_DEBUG, "Running PICMG GetDeviceLocator" );
+		memset(&req, 0, sizeof(req));
+		req.msg.netfn = IPMI_NETFN_PICMG;
+		req.msg.cmd = 0x01;
+		msg_data    = 0x00;
+		req.msg.data = &msg_data; 
+		req.msg.data_len = 1;
+		msg_data = 0;
+
+		rsp = intf->sendrecv(intf, &req);
+		if (rsp && !rsp->ccode) {
+			intf->my_addr = rsp->data[2];
+			intf->target_addr = intf->my_addr;
+			lprintf(LOG_DEBUG, "Discovered IPMB address = 0x%x", intf->my_addr);
+		}
+	}
+#endif
+
 	return intf->fd;
 }
 
@@ -119,6 +153,7 @@ ipmi_openipmi_send_cmd(struct ipmi_intf * intf, struct ipmi_rq * req)
 	};
 	struct ipmi_ipmb_addr ipmb_addr = {
 		addr_type:	IPMI_IPMB_ADDR_TYPE,
+		channel:	intf->target_channel & 0x0f,
 	};
 	struct ipmi_req _req;
 	static struct ipmi_rs rsp;
@@ -145,7 +180,7 @@ ipmi_openipmi_send_cmd(struct ipmi_intf * intf, struct ipmi_rq * req)
 	    intf->target_addr != intf->my_addr) {
 		/* use IPMB address if needed */
 		ipmb_addr.slave_addr = intf->target_addr;
-		_req.addr = (char *) &ipmb_addr;
+		_req.addr = (unsigned char *) &ipmb_addr;
 		_req.addr_len = sizeof(ipmb_addr);
 		lprintf(LOG_DEBUG, "Sending request to "
 			"IPMB target @ 0x%x", intf->target_addr);
@@ -153,7 +188,8 @@ ipmi_openipmi_send_cmd(struct ipmi_intf * intf, struct ipmi_rq * req)
 		/* otherwise use system interface */
 		lprintf(LOG_DEBUG+2, "Sending request to "
 			"System Interface");
-		_req.addr = (char *) &bmc_addr;
+		bmc_addr.lun = req->msg.lun;
+		_req.addr = (unsigned char *) &bmc_addr;
 		_req.addr_len = sizeof(bmc_addr);
 	}
 
@@ -187,7 +223,7 @@ ipmi_openipmi_send_cmd(struct ipmi_intf * intf, struct ipmi_rq * req)
 		return NULL;
 	}
 
-	recv.addr = (char *) &addr;
+	recv.addr = (unsigned char *) &addr;
 	recv.addr_len = sizeof(addr);
 	recv.msg.data = rsp.data;
 	recv.msg.data_len = sizeof(rsp.data);
